# SPDX-License-Identifier: GPL-2.0

import os
import socket
import struct
import threading
import unittest

from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PrivateKey

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path

expected_lines = [
    "default: ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAIFYQvN9a+toIB6jSs4zY7FMapZnHt80EKCUr/WhLwUum",
    "id1: ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAIFYQvN9a+toIB6jSs4zY7FMapZnHt80EKCUr/WhLwUum",
    "id2: ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAIFYQvN9a+toIB6jSs4zY7FMapZnHt80EKCUr/WhLwUum",
]


class TestBuildSsh(unittest.TestCase, RunSubprocessMixin):
    def test_build_ssh(self):
        """The build context can contain the ssh authentications that the image builder should
        use during image build. They can be either an array or a map.
        """

        compose_path = os.path.join(test_path(), "build_ssh/docker-compose.yml")
        sock_path = os.path.join(test_path(), "build_ssh/agent_dummy.sock")
        private_key_file = os.path.join(test_path(), "build_ssh/id_ed25519_dummy")

        agent = MockSSHAgent(private_key_file)

        try:
            # Set SSH_AUTH_SOCK because `default` expects it
            os.environ['SSH_AUTH_SOCK'] = sock_path

            # Start a mock SSH agent server
            agent.start_agent(sock_path)

            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "build",
                "test_build_ssh_map",
                "test_build_ssh_array",
            ])

            for test_image in [
                "test_build_ssh_map",
                "test_build_ssh_array",
            ]:
                out, _ = self.run_subprocess_assert_returncode([
                    podman_compose_path(),
                    "-f",
                    compose_path,
                    "run",
                    "--rm",
                    test_image,
                ])

                out = out.decode('utf-8')

                # Check if all lines are contained in the output
                self.assertTrue(
                    all(line in out for line in expected_lines),
                    f"Incorrect output for image {test_image}",
                )

        finally:
            # Now we send the stop command to gracefully shut down the server
            agent.stop_agent()

            if os.path.exists(sock_path):
                os.remove(sock_path)

            self.run_subprocess_assert_returncode([
                "podman",
                "rmi",
                "my-alpine-build-ssh-map",
                "my-alpine-build-ssh-array",
            ])


# SSH agent message types
SSH_AGENTC_REQUEST_IDENTITIES = 11
SSH_AGENT_IDENTITIES_ANSWER = 12
SSH_AGENT_FAILURE = 5
STOP_REQUEST = 0xFF


class MockSSHAgent:
    def __init__(self, private_key_path):
        self.sock_path = None
        self.server_sock = None
        self.running = threading.Event()
        self.keys = [self._load_ed25519_private_key(private_key_path)]
        self.agent_thread = None  # Thread to run the agent

    def _load_ed25519_private_key(self, private_key_path):
        """Load ED25519 private key from an OpenSSH private key file."""
        with open(private_key_path, 'rb') as key_file:
            private_key = serialization.load_ssh_private_key(key_file.read(), password=None)

        # Ensure it's an Ed25519 key
        if not isinstance(private_key, Ed25519PrivateKey):
            raise ValueError("Invalid key type, expected ED25519 private key.")

        # Get the public key corresponding to the private key
        public_key = private_key.public_key()

        # Serialize the public key to the OpenSSH format
        public_key_blob = public_key.public_bytes(
            encoding=serialization.Encoding.Raw, format=serialization.PublicFormat.Raw
        )

        # SSH key type "ssh-ed25519"
        key_type = b"ssh-ed25519"

        # Build the key blob (public key part for the agent)
        key_blob_full = (
            struct.pack(">I", len(key_type))
            + key_type  # Key type length + type
            + struct.pack(">I", len(public_key_blob))
            + public_key_blob  # Public key length + key blob
        )

        # Comment (empty)
        comment = ""

        return ("ssh-ed25519", key_blob_full, comment)

    def start_agent(self, sock_path):
        """Start the mock SSH agent and create a Unix domain socket."""
        self.sock_path = sock_path
        if os.path.exists(self.sock_path):
            os.remove(self.sock_path)

        self.server_sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self.server_sock.bind(self.sock_path)
        self.server_sock.listen(5)

        os.environ['SSH_AUTH_SOCK'] = self.sock_path

        self.running.set()  # Set the running event

        # Start a thread to accept client connections
        self.agent_thread = threading.Thread(target=self._accept_connections, daemon=True)
        self.agent_thread.start()

    def _accept_connections(self):
        """Accept and handle incoming connections."""
        while self.running.is_set():
            try:
                client_sock, _ = self.server_sock.accept()
                self._handle_client(client_sock)
            except Exception as e:
                print(f"Error accepting connection: {e}")

    def _handle_client(self, client_sock):
        """Handle a single client request (like ssh-add)."""
        try:
            # Read the message length (first 4 bytes)
            length_message = client_sock.recv(4)
            if not length_message:
                raise "no length message received"

            msg_len = struct.unpack(">I", length_message)[0]

            request_message = client_sock.recv(msg_len)

            # Check for STOP_REQUEST
            if request_message[0] == STOP_REQUEST:
                client_sock.close()
                self.running.clear()  # Stop accepting connections
                return

            # Check for SSH_AGENTC_REQUEST_IDENTITIES
            if request_message[0] == SSH_AGENTC_REQUEST_IDENTITIES:
                response = self._mock_list_keys_response()
                client_sock.sendall(response)
            else:
                print("Message not recognized")
                # Send failure if the message type is not recognized
                response = struct.pack(">I", 1) + struct.pack(">B", SSH_AGENT_FAILURE)
                client_sock.sendall(response)

        except OSError:
            print("Client socket error.")
            pass  # You can handle specific errors here if needed
        finally:
            client_sock.close()  # Ensure the client socket is closed

    def _mock_list_keys_response(self):
        """Create a mock response for ssh-add -l, listing keys."""

        # Start building the response
        response = struct.pack(">B", SSH_AGENT_IDENTITIES_ANSWER)  # Message type

        # Number of keys
        response += struct.pack(">I", len(self.keys))

        # For each key, append key blob and comment
        for key_type, key_blob, comment in self.keys:
            # Key blob length and content
            response += struct.pack(">I", len(key_blob)) + key_blob

            # Comment length and content
            comment_encoded = comment.encode()
            response += struct.pack(">I", len(comment_encoded)) + comment_encoded

        # Prefix the entire response with the total message length
        response = struct.pack(">I", len(response)) + response

        return response

    def stop_agent(self):
        """Stop the mock SSH agent."""
        if self.running.is_set():  # First check if the agent is running
            # Create a temporary connection to send the stop command
            with socket.socket(socket.AF_UNIX, socket.SOCK_STREAM) as client_sock:
                client_sock.connect(self.sock_path)  # Connect to the server

                stop_command = struct.pack(
                    ">B", STOP_REQUEST
                )  # Pack the stop command as a single byte

                # Send the message length first
                message_length = struct.pack(">I", len(stop_command))
                client_sock.sendall(message_length)  # Send the length first

                client_sock.sendall(stop_command)  # Send the stop command

            self.running.clear()  # Stop accepting new connections

            # Wait for the agent thread to finish
            if self.agent_thread:
                self.agent_thread.join()  # Wait for the thread to finish
                self.agent_thread = None  # Reset thread reference

            # Remove the socket file only after the server socket is closed
            if self.server_sock:  # Check if the server socket exists
                self.server_sock.close()  # Close the server socket
                os.remove(self.sock_path)



# SPDX-License-Identifier: GPL-2.0
# pylint: disable=protected-access

import io
import unittest
from typing import Union

from podman_compose import Podman


class DummyReader:
    def __init__(self, data: Union[list[bytes], None] = None):
        self.data = data or []

    async def readuntil(self, _: str) -> bytes:
        return self.data.pop(0)

    def at_eof(self) -> bool:
        return len(self.data) == 0


class TestComposeRunLogFormat(unittest.IsolatedAsyncioTestCase):
    def setUp(self) -> None:
        self.p = get_minimal_podman()
        self.buffer = io.StringIO()
        self.unicode_sample = b'\xe3\x81\x82\xe3\x81\x84\n'  # result of 'あい\n'.encode('utf-8')

    async def test_single_line_single_chunk(self) -> None:
        reader = DummyReader([b'hello, world\n'])
        await self.p._format_stream(reader, self.buffer, 'LL:')  # type: ignore[arg-type]
        self.assertEqual(self.buffer.getvalue(), 'LL: hello, world\n')

    async def test_empty(self) -> None:
        reader = DummyReader([])
        await self.p._format_stream(reader, self.buffer, 'LL:')  # type: ignore[arg-type]
        self.assertEqual(self.buffer.getvalue(), '')

    async def test_empty2(self) -> None:
        reader = DummyReader([b''])
        await self.p._format_stream(reader, self.buffer, 'LL:')  # type: ignore[arg-type]
        self.assertEqual(self.buffer.getvalue(), '')

    async def test_empty_line(self) -> None:
        reader = DummyReader([b'\n'])
        await self.p._format_stream(reader, self.buffer, 'LL:')  # type: ignore[arg-type]
        self.assertEqual(self.buffer.getvalue(), 'LL: \n')

    async def test_line_split(self) -> None:
        reader = DummyReader([b'hello,', b' world\n'])
        await self.p._format_stream(reader, self.buffer, 'LL:')  # type: ignore[arg-type]
        self.assertEqual(self.buffer.getvalue(), 'LL: hello, world\n')

    async def test_two_lines_in_one_chunk(self) -> None:
        reader = DummyReader([b'hello\nbye\n'])
        await self.p._format_stream(reader, self.buffer, 'LL:')  # type: ignore[arg-type]
        self.assertEqual(self.buffer.getvalue(), 'LL: hello\nLL: bye\n')

    async def test_double_blank(self) -> None:
        reader = DummyReader([b'hello\n\n\nbye\n'])
        await self.p._format_stream(reader, self.buffer, 'LL:')  # type: ignore[arg-type]
        self.assertEqual(self.buffer.getvalue(), 'LL: hello\nLL: \nLL: \nLL: bye\n')

    async def test_no_new_line_at_end(self) -> None:
        reader = DummyReader([b'hello\nbye'])
        await self.p._format_stream(reader, self.buffer, 'LL:')  # type: ignore[arg-type]
        self.assertEqual(self.buffer.getvalue(), 'LL: hello\nLL: bye\n')

    async def test_split_multibyte(self) -> None:
        string = self.unicode_sample
        mid = 4
        reader = DummyReader([string[:mid], string[mid:]])
        await self.p._format_stream(reader, self.buffer, 'LL:')  # type: ignore[arg-type]
        self.assertEqual(self.buffer.getvalue(), 'LL: あい\n')

    async def test_incomplete_multibyte_at_end(self) -> None:
        string = self.unicode_sample
        mid = 4
        reader = DummyReader([string[:mid]])
        await self.p._format_stream(reader, self.buffer, 'LL:')  # type: ignore[arg-type]
        self.assertEqual(self.buffer.getvalue(), 'LL: あ\n')

    async def test_incomplete_multibyte_at_beginning(self) -> None:
        string = self.unicode_sample
        mid = 4
        reader = DummyReader([string[mid:]])
        with self.assertRaises(UnicodeDecodeError):
            await self.p._format_stream(reader, self.buffer, 'LL:')  # type: ignore[arg-type]


def get_minimal_podman() -> Podman:
    return Podman(None)  # type: ignore[arg-type]



# SPDX-License-Identifier: GPL-2.0
# pylint: disable=redefined-outer-name
import unittest

from podman_compose import parse_short_mount


class ParseShortMountTests(unittest.TestCase):
    def test_multi_propagation(self) -> None:
        self.assertEqual(
            parse_short_mount("/foo/bar:/baz:U,Z", "/"),
            {
                "type": "bind",
                "source": "/foo/bar",
                "target": "/baz",
                "bind": {
                    "propagation": "U,Z",
                },
            },
        )



# SPDX-License-Identifier: GPL-2.0


import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


class TestUlimit(unittest.TestCase, RunSubprocessMixin):
    def test_ulimit(self) -> None:
        compose_path = os.path.join(test_path(), "ulimit/docker-compose.yaml")
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "up",
                "-d",
            ])

            out, _ = self.run_subprocess_assert_returncode([
                "podman",
                "logs",
                "ulimit_ulimit1_1",
            ])
            split_output = out.strip(b"\n").split(b"\n")

            # trow away system specific default ulimit values
            output_part = [
                el
                for el in split_output
                if not el.startswith(b"soft process") and not el.startswith(b"hard process")
            ]
            self.assertEqual(
                output_part,
                [
                    b"soft nofile limit 1001",
                    b"hard nofile limit 1001",
                ],
            )

            out, _ = self.run_subprocess_assert_returncode([
                "podman",
                "logs",
                "ulimit_ulimit2_1",
            ])
            self.assertEqual(
                out,
                b"soft process limit 1002\nhard process limit 2002\nsoft nofile limit 1002\n"
                b"hard nofile limit 1002\n",
            )

            out, _ = self.run_subprocess_assert_returncode([
                "podman",
                "logs",
                "ulimit_ulimit3_1",
            ])
            self.assertEqual(
                out,
                b"soft process limit 1003\nhard process limit 2003\nsoft nofile limit 1003\n"
                b"hard nofile limit 1003\n",
            )
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0

import os
import unittest
from unittest import mock

from podman_compose import container_to_build_args


def create_compose_mock(project_name='test_project_name'):
    compose = mock.Mock()
    compose.project_name = project_name
    compose.dirname = 'test_dirname'
    compose.container_names_by_service.get = mock.Mock(return_value=None)
    compose.prefer_volume_over_mount = False
    compose.default_net = None
    compose.networks = {}
    compose.x_podman = {}
    return compose


def get_minimal_container():
    return {
        'name': 'project_name_service_name1',
        'service_name': 'service_name',
        'image': 'new-image',
        'build': {},
    }


def get_minimal_args():
    args = mock.Mock()
    args.build_arg = []
    args.pull = None
    return args


class TestContainerToBuildArgs(unittest.TestCase):
    def test_minimal(self):
        c = create_compose_mock()

        cnt = get_minimal_container()
        args = get_minimal_args()

        args = container_to_build_args(c, cnt, args, lambda path: True)
        self.assertEqual(
            args,
            [
                '-f',
                'Containerfile',
                '-t',
                'new-image',
                '--no-cache',
                '.',
            ],
        )

    def test_platform(self):
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['platform'] = 'linux/amd64'
        args = get_minimal_args()

        args = container_to_build_args(c, cnt, args, lambda path: True)
        self.assertEqual(
            args,
            [
                '-f',
                'Containerfile',
                '-t',
                'new-image',
                '--platform',
                'linux/amd64',
                '--no-cache',
                '.',
            ],
        )

    def test_tags(self):
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['build']['tags'] = ['some-tag1', 'some-tag2:2']
        args = get_minimal_args()

        args = container_to_build_args(c, cnt, args, lambda path: True)
        self.assertEqual(
            args,
            [
                '-f',
                'Containerfile',
                '-t',
                'new-image',
                '-t',
                'some-tag1',
                '-t',
                'some-tag2:2',
                '--no-cache',
                '.',
            ],
        )

    def test_labels(self):
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['build']['labels'] = ['some-label1', 'some-label2.2']
        args = get_minimal_args()

        args = container_to_build_args(c, cnt, args, lambda path: True)
        self.assertEqual(
            args,
            [
                '-f',
                'Containerfile',
                '-t',
                'new-image',
                '--label',
                'some-label1',
                '--label',
                'some-label2.2',
                '--no-cache',
                '.',
            ],
        )

    def test_caches(self):
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['build']['cache_from'] = ['registry/image1', 'registry/image2']
        cnt['build']['cache_to'] = ['registry/image3', 'registry/image4']
        args = get_minimal_args()

        args = container_to_build_args(c, cnt, args, lambda path: True)
        self.assertEqual(
            args,
            [
                '-f',
                'Containerfile',
                '-t',
                'new-image',
                '--no-cache',
                '--cache-from',
                'registry/image1',
                '--cache-from',
                'registry/image2',
                '--cache-to',
                'registry/image3',
                '--cache-to',
                'registry/image4',
                '.',
            ],
        )

    def test_dockerfile_inline(self):
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['build']['dockerfile_inline'] = "FROM busybox\nRUN echo 'hello world'"
        args = get_minimal_args()

        cleanup_callbacks = []
        args = container_to_build_args(
            c, cnt, args, lambda path: True, cleanup_callbacks=cleanup_callbacks
        )

        temp_dockerfile = args[args.index("-f") + 1]
        self.assertTrue(os.path.exists(temp_dockerfile))

        with open(temp_dockerfile) as file:
            contents = file.read()
            self.assertEqual(contents, "FROM busybox\n" + "RUN echo 'hello world'")

        for c in cleanup_callbacks:
            c()
        self.assertFalse(os.path.exists(temp_dockerfile))

    def test_context_git_url(self):
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['build']['context'] = "https://github.com/test_repo.git"
        args = get_minimal_args()

        args = container_to_build_args(c, cnt, args, lambda path: False)
        self.assertEqual(
            args,
            [
                '-t',
                'new-image',
                '--no-cache',
                'https://github.com/test_repo.git',
            ],
        )

    def test_context_invalid_git_url_git_is_not_prefix(self):
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['build']['context'] = "not_prefix://github.com/test_repo"
        args = get_minimal_args()

        with self.assertRaises(OSError):
            container_to_build_args(c, cnt, args, lambda path: False)

    def test_build_ssh_absolute_path(self):
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['build']['ssh'] = ["id1=/test1"]
        args = get_minimal_args()

        args = container_to_build_args(c, cnt, args, lambda path: True)
        self.assertEqual(
            args,
            [
                '-f',
                'Containerfile',
                '-t',
                'new-image',
                '--ssh',
                'id1=/test1',
                '--no-cache',
                '.',
            ],
        )

    def test_build_ssh_relative_path(self):
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['build']['ssh'] = ["id1=id1/test1"]
        args = get_minimal_args()

        args = container_to_build_args(c, cnt, args, lambda path: True)
        self.assertEqual(
            args,
            [
                '-f',
                'Containerfile',
                '-t',
                'new-image',
                '--ssh',
                'id1=test_dirname/id1/test1',
                '--no-cache',
                '.',
            ],
        )

    def test_build_ssh_working_dir(self):
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['build']['ssh'] = ["id1=./test1"]
        args = get_minimal_args()

        args = container_to_build_args(c, cnt, args, lambda path: True)
        self.assertEqual(
            args,
            [
                '-f',
                'Containerfile',
                '-t',
                'new-image',
                '--ssh',
                'id1=test_dirname/./test1',
                '--no-cache',
                '.',
            ],
        )

    @mock.patch.dict(os.environ, {"HOME": "/home/user"}, clear=True)
    def test_build_ssh_path_home_dir(self):
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['build']['ssh'] = ["id1=~/test1"]
        args = get_minimal_args()

        args = container_to_build_args(c, cnt, args, lambda path: True)
        self.assertEqual(
            args,
            [
                '-f',
                'Containerfile',
                '-t',
                'new-image',
                '--ssh',
                'id1=/home/user/test1',
                '--no-cache',
                '.',
            ],
        )

    def test_build_ssh_map(self):
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['build']['ssh'] = {"id1": "test1", "id2": "test2"}
        args = get_minimal_args()

        args = container_to_build_args(c, cnt, args, lambda path: True)
        self.assertEqual(
            args,
            [
                '-f',
                'Containerfile',
                '-t',
                'new-image',
                '--ssh',
                'id1=test_dirname/test1',
                '--ssh',
                'id2=test_dirname/test2',
                '--no-cache',
                '.',
            ],
        )

    def test_build_ssh_array(self):
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['build']['ssh'] = ['id1=test1', 'id2=test2']
        args = get_minimal_args()

        args = container_to_build_args(c, cnt, args, lambda path: True)
        self.assertEqual(
            args,
            [
                '-f',
                'Containerfile',
                '-t',
                'new-image',
                '--ssh',
                'id1=test_dirname/test1',
                '--ssh',
                'id2=test_dirname/test2',
                '--no-cache',
                '.',
            ],
        )

    def test_pull_always(self):
        c = create_compose_mock()
        cnt = get_minimal_container()
        args = get_minimal_args()
        args.pull = 'always'

        args = container_to_build_args(c, cnt, args, lambda path: True)
        self.assertEqual(
            args,
            [
                '-f',
                'Containerfile',
                '-t',
                'new-image',
                '--no-cache',
                '--pull=always',
                '.',
            ],
        )

    def test_containerfile_in_context(self):
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['build']['context'] = "./subdir"
        args = get_minimal_args()
        args = container_to_build_args(c, cnt, args, lambda path: True)
        self.assertEqual(
            args,
            [
                '-f',
                'subdir/Containerfile',
                '-t',
                'new-image',
                '--no-cache',
                './subdir',
            ],
        )

    def test_build_environment_secret_no_target(self):
        c = create_compose_mock()
        c.declared_secrets = {'my_secret': {'environment': 'MY_VAR'}}
        cnt = get_minimal_container()
        cnt['build']['secrets'] = ['my_secret']
        args = get_minimal_args()
        args = container_to_build_args(c, cnt, args, lambda path: True)
        self.assertEqual(
            args,
            [
                '-f',
                'Containerfile',
                '-t',
                'new-image',
                '--secret',
                'id=my_secret,env=MY_VAR',
                '--no-cache',
                '.',
            ],
        )

    def test_build_environment_secret_with_target(self):
        c = create_compose_mock()
        c.declared_secrets = {'my_secret': {'environment': 'MY_VAR'}}
        cnt = get_minimal_container()
        cnt['build']['secrets'] = [{'source': 'my_secret', 'target': 'custom_id'}]
        args = get_minimal_args()
        args = container_to_build_args(c, cnt, args, lambda path: True)
        self.assertEqual(
            args,
            [
                '-f',
                'Containerfile',
                '-t',
                'new-image',
                '--secret',
                'id=custom_id,env=MY_VAR',
                '--no-cache',
                '.',
            ],
        )



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    """Returns the path to the compose file used for this test module"""
    base_path = os.path.join(test_path(), "extra_hosts")
    return os.path.join(base_path, "docker-compose.yml")


class TestComposeExtraHosts(unittest.TestCase, RunSubprocessMixin):
    def test_extra_hosts(self) -> None:
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "build",
                "--no-cache",
            ])

            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "up",
                "-d",
            ])

        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0

import json
import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(os.path.join(test_path(), "ipam_default"), "docker-compose.yaml")


class TestComposeIpamDefault(unittest.TestCase, RunSubprocessMixin):
    def test_ipam_default(self) -> None:
        try:
            self.run_subprocess_assert_returncode(
                [podman_compose_path(), "-f", compose_yaml_path(), "up", "-d"],
            )

            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "logs",
            ])
            # when container is created, its command echoes 'ipamtest'
            # BUG: figure out why echo is called twice
            self.assertIn("ipamtest", str(output))

            output, _ = self.run_subprocess_assert_returncode(
                [
                    "podman",
                    "inspect",
                    "ipam_default_testipam_1",
                ],
            )
            network_info = json.loads(output.decode('utf-8'))[0]
            network_name = next(iter(network_info["NetworkSettings"]["Networks"].keys()))

            output, _ = self.run_subprocess_assert_returncode([
                "podman",
                "network",
                "inspect",
                f"{network_name}",
            ])
            network_info = json.loads(output.decode('utf-8'))[0]
            # bridge is the default network driver
            self.assertEqual(network_info['driver'], "bridge")
            self.assertEqual(network_info['ipam_options'], {'driver': 'host-local'})
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0


"""Test how ulimits are applied in podman-compose build."""

import os
import subprocess
import unittest

from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path():
    """ "Returns the path to the compose file used for this test module"""
    return os.path.join(test_path(), "ulimit_build")


class TestComposeBuildUlimits(unittest.TestCase):
    def test_build_ulimits_ulimit1(self):
        """podman build should receive and apply limits when building service ulimit1"""

        cmd = (
            "coverage",
            "run",
            podman_compose_path(),
            "--verbose",
            "-f",
            os.path.join(compose_yaml_path(), "docker-compose.yaml"),
            "build",
            "--no-cache",
            "ulimit1",
        )
        p = subprocess.run(
            cmd, stdout=subprocess.PIPE, check=False, stderr=subprocess.STDOUT, text=True
        )

        self.assertEqual(p.returncode, 0)
        self.assertIn("--ulimit nofile=1001", p.stdout)
        self.assertIn("soft nofile limit: 1001", p.stdout)
        self.assertIn("hard nofile limit: 1001", p.stdout)

    def test_build_ulimits_ulimit2(self):
        """podman build should receive and apply limits when building service ulimit2"""

        cmd = (
            "coverage",
            "run",
            podman_compose_path(),
            "--verbose",
            "-f",
            os.path.join(compose_yaml_path(), "docker-compose.yaml"),
            "build",
            "--no-cache",
            "ulimit2",
        )
        p = subprocess.run(
            cmd, stdout=subprocess.PIPE, check=False, stderr=subprocess.STDOUT, text=True
        )

        self.assertEqual(p.returncode, 0)
        self.assertIn("--ulimit nofile=1002", p.stdout)
        self.assertIn("--ulimit nproc=1002:2002", p.stdout)
        self.assertIn("soft process limit: 1002", p.stdout)
        self.assertIn("hard process limit: 2002", p.stdout)
        self.assertIn("soft nofile limit: 1002", p.stdout)
        self.assertIn("hard nofile limit: 1002", p.stdout)

    def test_build_ulimits_ulimit3(self):
        """podman build should receive and apply limits when building service ulimit3"""

        cmd = (
            "coverage",
            "run",
            podman_compose_path(),
            "--verbose",
            "-f",
            os.path.join(compose_yaml_path(), "docker-compose.yaml"),
            "build",
            "--no-cache",
            "ulimit3",
        )
        p = subprocess.run(
            cmd, stdout=subprocess.PIPE, check=False, stderr=subprocess.STDOUT, text=True
        )

        self.assertEqual(p.returncode, 0)
        self.assertIn("--ulimit nofile=1003", p.stdout)
        self.assertIn("--ulimit nproc=1003:2003", p.stdout)
        self.assertIn("soft process limit: 1003", p.stdout)
        self.assertIn("hard process limit: 2003", p.stdout)
        self.assertIn("soft nofile limit: 1003", p.stdout)
        self.assertIn("hard nofile limit: 1003", p.stdout)



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(os.path.join(test_path(), "exit_from"), "docker-compose.yaml")


class TestComposeExitFrom(unittest.TestCase, RunSubprocessMixin):
    def test_exit_code_sh1(self) -> None:
        try:
            self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "up",
                    "--exit-code-from=sh1",
                ],
                1,
            )
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])

    def test_exit_code_sh2(self) -> None:
        try:
            self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "up",
                    "--exit-code-from=sh2",
                ],
                2,
            )
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])

    def test_podman_compose_exit_from(self) -> None:
        up_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            compose_yaml_path(),
            "up",
        ]

        self.run_subprocess_assert_returncode(up_cmd + ["--exit-code-from", "sh1"], 1)
        self.run_subprocess_assert_returncode(up_cmd + ["--exit-code-from", "sh2"], 2)



# SPDX-License-Identifier: GPL-2.0

"""
test_podman_compose_up_down.py

Tests the podman compose up and down commands used to create and remove services.
"""

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


class TestPodmanCompose(unittest.TestCase, RunSubprocessMixin):
    up_cmd = [
        "coverage",
        "run",
        podman_compose_path(),
        "-f",
        os.path.join(test_path(), "up_down", "docker-compose.yml"),
        "up",
        "-d",
        "--force-recreate",
    ]

    def setUp(self) -> None:
        """
        Retag the debian image before each test to not mess with the other integration tests when
        testing the `--rmi` argument
        """
        tag_cmd = [
            "podman",
            "tag",
            "docker.io/library/debian:bookworm-slim",
            "docker.io/library/debian:up-down-test",
        ]
        self.run_subprocess_assert_returncode(tag_cmd)

    @classmethod
    def tearDownClass(cls) -> None:
        """
        Ensures that the images that were created for this tests will be removed
        """
        rmi_cmd = [
            "podman",
            "rmi",
            "--force",
            "--ignore",
            "podman-compose-up-down-test",
            "docker.io/library/debian:up-down-test",
        ]
        cls().run_subprocess_assert_returncode(rmi_cmd)

    def test_down(self) -> None:
        down_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            os.path.join(test_path(), "up_down", "docker-compose.yml"),
            "down",
            "--timeout",
            "0",
        ]

        try:
            self.run_subprocess_assert_returncode(self.up_cmd)
            self.run_subprocess_assert_returncode([
                "podman",
                "container",
                "exists",
                "up_down_web1_1",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "container",
                "exists",
                "up_down_web2_1",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "volume",
                "exists",
                "up_down_web1_vol",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "volume",
                "exists",
                "up_down_web2_vol",
            ])
        finally:
            self.run_subprocess_assert_returncode(down_cmd)

        self.run_subprocess_assert_returncode(
            ["podman", "container", "exists", "up_down_web1_1"], 1
        )
        self.run_subprocess_assert_returncode(
            ["podman", "container", "exists", "up_down_web2_1"], 1
        )
        self.run_subprocess_assert_returncode(["podman", "volume", "exists", "up_down_web1_vol"])
        self.run_subprocess_assert_returncode(["podman", "volume", "exists", "up_down_web2_vol"])
        self.run_subprocess_assert_returncode([
            "podman",
            "image",
            "exists",
            "podman-compose-up-down-test",
        ])
        self.run_subprocess_assert_returncode([
            "podman",
            "image",
            "exists",
            "docker.io/library/debian:up-down-test",
        ])

    def test_down_with_volumes(self) -> None:
        down_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            os.path.join(test_path(), "up_down", "docker-compose.yml"),
            "down",
            "--volumes",
            "--timeout",
            "0",
        ]

        try:
            self.run_subprocess_assert_returncode(self.up_cmd)
            self.run_subprocess_assert_returncode([
                "podman",
                "container",
                "exists",
                "up_down_web1_1",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "container",
                "exists",
                "up_down_web2_1",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "volume",
                "exists",
                "up_down_web1_vol",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "volume",
                "exists",
                "up_down_web2_vol",
            ])
        finally:
            self.run_subprocess_assert_returncode(down_cmd)

        self.run_subprocess_assert_returncode(
            ["podman", "container", "exists", "up_down_web1_1"], 1
        )
        self.run_subprocess_assert_returncode(
            ["podman", "container", "exists", "up_down_web2_1"], 1
        )
        self.run_subprocess_assert_returncode(["podman", "volume", "exists", "up_down_web1_vol"], 1)
        self.run_subprocess_assert_returncode(["podman", "volume", "exists", "up_down_web2_vol"], 1)
        self.run_subprocess_assert_returncode([
            "podman",
            "image",
            "exists",
            "podman-compose-up-down-test",
        ])
        self.run_subprocess_assert_returncode([
            "podman",
            "image",
            "exists",
            "docker.io/library/debian:up-down-test",
        ])

    def test_down_without_orphans(self) -> None:
        down_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            os.path.join(test_path(), "up_down", "docker-compose-orphans.yml"),
            "down",
            "--volumes",
            "--timeout",
            "0",
        ]

        try:
            self.run_subprocess_assert_returncode(self.up_cmd)
            self.run_subprocess_assert_returncode([
                "podman",
                "container",
                "exists",
                "up_down_web1_1",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "container",
                "exists",
                "up_down_web2_1",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "volume",
                "exists",
                "up_down_web1_vol",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "volume",
                "exists",
                "up_down_web2_vol",
            ])
        finally:
            self.run_subprocess_assert_returncode(down_cmd)

        self.run_subprocess_assert_returncode(
            ["podman", "container", "exists", "up_down_web1_1"], 1
        )
        self.run_subprocess_assert_returncode(["podman", "container", "exists", "up_down_web2_1"])
        self.run_subprocess_assert_returncode(["podman", "volume", "exists", "up_down_web1_vol"], 1)
        self.run_subprocess_assert_returncode(["podman", "volume", "exists", "up_down_web2_vol"])
        self.run_subprocess_assert_returncode([
            "podman",
            "image",
            "exists",
            "podman-compose-up-down-test",
        ])
        self.run_subprocess_assert_returncode([
            "podman",
            "image",
            "exists",
            "docker.io/library/debian:up-down-test",
        ])

        # Cleanup orphaned container
        down_all_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            os.path.join(test_path(), "up_down", "docker-compose.yml"),
            "down",
            "--volumes",
            "--timeout",
            "0",
        ]
        self.run_subprocess_assert_returncode(down_all_cmd)
        self.run_subprocess_assert_returncode(
            ["podman", "container", "exists", "up_down_web2_1"], 1
        )
        self.run_subprocess_assert_returncode(["podman", "volume", "exists", "up_down_web2_vol"], 1)

    def test_down_with_orphans(self) -> None:
        down_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            os.path.join(test_path(), "up_down", "docker-compose-orphans.yml"),
            "down",
            "--volumes",
            "--remove-orphans",
            "--timeout",
            "0",
        ]

        try:
            self.run_subprocess_assert_returncode(self.up_cmd)
            self.run_subprocess_assert_returncode([
                "podman",
                "container",
                "exists",
                "up_down_web1_1",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "container",
                "exists",
                "up_down_web2_1",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "volume",
                "exists",
                "up_down_web1_vol",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "volume",
                "exists",
                "up_down_web2_vol",
            ])
        finally:
            self.run_subprocess_assert_returncode(down_cmd)

        self.run_subprocess_assert_returncode(
            ["podman", "container", "exists", "up_down_web1_1"], 1
        )
        self.run_subprocess_assert_returncode(
            ["podman", "container", "exists", "up_down_web2_1"], 1
        )
        self.run_subprocess_assert_returncode(["podman", "volume", "exists", "up_down_web1_vol"], 1)
        self.run_subprocess_assert_returncode(["podman", "volume", "exists", "up_down_web2_vol"], 1)
        self.run_subprocess_assert_returncode([
            "podman",
            "image",
            "exists",
            "podman-compose-up-down-test",
        ])
        self.run_subprocess_assert_returncode([
            "podman",
            "image",
            "exists",
            "docker.io/library/debian:up-down-test",
        ])

    def test_down_with_images_default(self) -> None:
        down_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            os.path.join(test_path(), "up_down", "docker-compose.yml"),
            "down",
            "--rmi",
            "--timeout",
            "0",
        ]

        try:
            self.run_subprocess_assert_returncode(self.up_cmd)
            self.run_subprocess_assert_returncode([
                "podman",
                "container",
                "exists",
                "up_down_web1_1",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "container",
                "exists",
                "up_down_web2_1",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "volume",
                "exists",
                "up_down_web1_vol",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "volume",
                "exists",
                "up_down_web2_vol",
            ])
        finally:
            self.run_subprocess_assert_returncode(down_cmd)

        self.run_subprocess_assert_returncode(
            ["podman", "container", "exists", "up_down_web1_1"], 1
        )
        self.run_subprocess_assert_returncode(
            ["podman", "container", "exists", "up_down_web2_1"], 1
        )
        self.run_subprocess_assert_returncode(["podman", "volume", "exists", "up_down_web1_vol"])
        self.run_subprocess_assert_returncode(["podman", "volume", "exists", "up_down_web2_vol"])
        self.run_subprocess_assert_returncode(
            ["podman", "image", "exists", "podman-compose-up-down-test"], 1
        )
        self.run_subprocess_assert_returncode(
            ["podman", "image", "exists", "docker.io/library/debian:up-down-test"], 1
        )

    def test_down_with_images_all(self) -> None:
        down_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            os.path.join(test_path(), "up_down", "docker-compose.yml"),
            "down",
            "--rmi",
            "all",
            "--timeout",
            "0",
        ]

        try:
            self.run_subprocess_assert_returncode(self.up_cmd)
            self.run_subprocess_assert_returncode([
                "podman",
                "container",
                "exists",
                "up_down_web1_1",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "container",
                "exists",
                "up_down_web2_1",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "volume",
                "exists",
                "up_down_web1_vol",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "volume",
                "exists",
                "up_down_web2_vol",
            ])
        finally:
            self.run_subprocess_assert_returncode(down_cmd)

        self.run_subprocess_assert_returncode(
            ["podman", "container", "exists", "up_down_web1_1"], 1
        )
        self.run_subprocess_assert_returncode(
            ["podman", "container", "exists", "up_down_web2_1"], 1
        )
        self.run_subprocess_assert_returncode(["podman", "volume", "exists", "up_down_web1_vol"])
        self.run_subprocess_assert_returncode(["podman", "volume", "exists", "up_down_web2_vol"])
        self.run_subprocess_assert_returncode(
            ["podman", "image", "exists", "podman-compose-up-down-test"], 1
        )
        self.run_subprocess_assert_returncode(
            ["podman", "image", "exists", "docker.io/library/debian:up-down-test"], 1
        )

    def test_down_with_images_all_and_orphans(self) -> None:
        down_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            os.path.join(test_path(), "up_down", "docker-compose-orphans.yml"),
            "down",
            "--volumes",
            "--remove-orphans",
            "--rmi",
            "all",
            "--timeout",
            "0",
        ]

        try:
            self.run_subprocess_assert_returncode(self.up_cmd)
            self.run_subprocess_assert_returncode([
                "podman",
                "container",
                "exists",
                "up_down_web1_1",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "container",
                "exists",
                "up_down_web2_1",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "volume",
                "exists",
                "up_down_web1_vol",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "volume",
                "exists",
                "up_down_web2_vol",
            ])
        finally:
            self.run_subprocess_assert_returncode(down_cmd)

        self.run_subprocess_assert_returncode(
            ["podman", "container", "exists", "up_down_web1_1"], 1
        )
        self.run_subprocess_assert_returncode(
            ["podman", "container", "exists", "up_down_web2_1"], 1
        )
        self.run_subprocess_assert_returncode(["podman", "volume", "exists", "up_down_web1_vol"], 1)
        self.run_subprocess_assert_returncode(["podman", "volume", "exists", "up_down_web2_vol"], 1)
        self.run_subprocess_assert_returncode(
            ["podman", "image", "exists", "podman-compose-up-down-test"], 1
        )
        self.run_subprocess_assert_returncode(
            ["podman", "image", "exists", "docker.io/library/debian:up-down-test"], 1
        )

    def test_down_with_images_local(self) -> None:
        down_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            os.path.join(test_path(), "up_down", "docker-compose.yml"),
            "down",
            "--rmi",
            "local",
            "--timeout",
            "0",
        ]

        try:
            self.run_subprocess_assert_returncode(self.up_cmd)
            self.run_subprocess_assert_returncode([
                "podman",
                "container",
                "exists",
                "up_down_web1_1",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "container",
                "exists",
                "up_down_web2_1",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "volume",
                "exists",
                "up_down_web1_vol",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "volume",
                "exists",
                "up_down_web2_vol",
            ])
        finally:
            self.run_subprocess_assert_returncode(down_cmd)

        self.run_subprocess_assert_returncode(
            ["podman", "container", "exists", "up_down_web1_1"], 1
        )
        self.run_subprocess_assert_returncode(
            ["podman", "container", "exists", "up_down_web2_1"], 1
        )
        self.run_subprocess_assert_returncode(["podman", "volume", "exists", "up_down_web1_vol"])
        self.run_subprocess_assert_returncode(["podman", "volume", "exists", "up_down_web2_vol"])
        self.run_subprocess_assert_returncode(
            ["podman", "image", "exists", "podman-compose-up-down-test"], 1
        )
        self.run_subprocess_assert_returncode([
            "podman",
            "image",
            "exists",
            "docker.io/library/debian:up-down-test",
        ])



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from packaging import version

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import get_podman_version
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path(test_ref_folder: str) -> str:
    return os.path.join(test_path(), "service_scale", test_ref_folder, "docker-compose.yml")


@unittest.skipIf(get_podman_version() >= version.parse("5.0.0"), "Breaks as of podman-5.4.2.")
class TestComposeScale(unittest.TestCase, RunSubprocessMixin):
    # scale-up using `scale` parameter in docker-compose.yml
    def test_scaleup_scale_parameter(self) -> None:
        try:
            output, _, return_code = self.run_subprocess([
                podman_compose_path(),
                "-f",
                compose_yaml_path("scaleup_scale_parameter"),
                "up",
                "-d",
            ])
            self.assertEqual(return_code, 0)
            output, _, return_code = self.run_subprocess([
                podman_compose_path(),
                "-f",
                compose_yaml_path("scaleup_scale_parameter"),
                "ps",
                "-q",
            ])
            self.assertEqual(len(output.splitlines()), 2)
        finally:
            self.run_subprocess_assert_returncode([
                "podman",
                "rm",
                "--force",
                "-t",
                "0",
                "podman-compose_service1_1",
                "podman-compose_service1_2",
            ])
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path("scaleup_scale_parameter"),
                "down",
                "-t",
                "0",
            ])

    # scale-up using `deploy => replicas` parameter in docker-compose.yml
    def test_scaleup_deploy_replicas_parameter(self) -> None:
        try:
            output, _, return_code = self.run_subprocess([
                podman_compose_path(),
                "-f",
                compose_yaml_path('scaleup_deploy_replicas_parameter'),
                "up",
                "-d",
            ])
            self.assertEqual(return_code, 0)
            output, _, return_code = self.run_subprocess([
                podman_compose_path(),
                "-f",
                compose_yaml_path("scaleup_deploy_replicas_parameter"),
                "ps",
                "-q",
            ])
            self.assertEqual(len(output.splitlines()), 3)
        finally:
            self.run_subprocess_assert_returncode([
                "podman",
                "rm",
                "--force",
                "-t",
                "0",
                "podman-compose_service1_1",
                "podman-compose_service1_2",
                "podman-compose_service1_3",
            ])
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path('scaleup_deploy_replicas_parameter'),
                "down",
                "-t",
                "0",
            ])

    # scale-up using `--scale <SERVICE>=<number of replicas>` argument in CLI
    def test_scaleup_cli(self) -> None:
        try:
            output, _, return_code = self.run_subprocess([
                podman_compose_path(),
                "-f",
                compose_yaml_path('scaleup_cli'),
                "up",
                "-d",
            ])
            self.assertEqual(return_code, 0)
            output, _, return_code = self.run_subprocess([
                podman_compose_path(),
                "-f",
                compose_yaml_path('scaleup_cli'),
                "up",
                "-d",
                "--scale",
                "service1=4",
            ])
            self.assertEqual(return_code, 0)

            output, _, return_code = self.run_subprocess([
                podman_compose_path(),
                "-f",
                compose_yaml_path('scaleup_cli'),
                "ps",
                "-q",
            ])
            self.assertEqual(len(output.splitlines()), 4)
        finally:
            self.run_subprocess_assert_returncode([
                "podman",
                "rm",
                "--force",
                "-t",
                "0",
                "podman-compose_service1_1",
                "podman-compose_service1_2",
                "podman-compose_service1_3",
                "podman-compose_service1_4",
            ])
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path('scaleup_cli'),
                "down",
                "-t",
                "0",
            ])



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from parameterized import parameterized

from podman_compose import container_to_args
from tests.unit.test_container_to_args import create_compose_mock
from tests.unit.test_container_to_args import get_minimal_container


def repo_root() -> str:
    return os.path.dirname(os.path.dirname(os.path.dirname(__file__)))


class TestContainerToArgsSecrets(unittest.IsolatedAsyncioTestCase):
    async def test_pass_secret_as_env_variable(self) -> None:
        c = create_compose_mock()
        c.declared_secrets = {
            "my_secret": {"external": "true"}  # must have external or name value
        }

        cnt = get_minimal_container()
        cnt["secrets"] = [
            {
                "source": "my_secret",
                "target": "ENV_SECRET",
                "type": "env",
            },
        ]

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--secret",
                "my_secret,type=env,target=ENV_SECRET",
                "busybox",
            ],
        )

    async def test_secret_as_env_external_true_has_no_name(self) -> None:
        c = create_compose_mock()
        c.declared_secrets = {
            "my_secret": {
                "name": "my_secret",  # must have external or name value
            }
        }

        cnt = get_minimal_container()
        cnt["_service"] = "test-service"
        cnt["secrets"] = [
            {
                "source": "my_secret",
                "target": "ENV_SECRET",
                "type": "env",
            }
        ]

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--secret",
                "my_secret,type=env,target=ENV_SECRET",
                "busybox",
            ],
        )

    async def test_pass_secret_as_env_variable_no_external(self) -> None:
        c = create_compose_mock()
        c.declared_secrets = {
            "my_secret": {}  # must have external or name value
        }

        cnt = get_minimal_container()
        cnt["_service"] = "test-service"
        cnt["secrets"] = [
            {
                "source": "my_secret",
                "target": "ENV_SECRET",
                "type": "env",
            }
        ]

        with self.assertRaises(ValueError) as context:
            await container_to_args(c, cnt)
        self.assertIn('ERROR: unparsable secret: ', str(context.exception))

    @parameterized.expand([
        (
            "secret_no_name",
            {"my_secret": "my_secret_name", "external": "true"},
            {},  # must have a name
        ),
        (
            "no_secret_name_in_declared_secrets",
            {},  # must have a name
            {
                "source": "my_secret_name",
            },
        ),
        (
            "secret_name_does_not_match_declared_secrets_name",
            {
                "wrong_name": "my_secret_name",
            },
            {
                "source": "name",  # secret name must match the one in declared_secrets
            },
        ),
        (
            "secret_name_empty_string",
            {"": "my_secret_name"},
            {
                "source": "",  # can not be empty string
            },
        ),
    ])
    async def test_secret_name(
        self, test_name: str, declared_secrets: dict, add_to_minimal_container: dict
    ) -> None:
        c = create_compose_mock()
        c.declared_secrets = declared_secrets

        cnt = get_minimal_container()
        cnt["_service"] = "test-service"
        cnt["secrets"] = [add_to_minimal_container]

        with self.assertRaises(ValueError) as context:
            await container_to_args(c, cnt)
        self.assertIn('ERROR: undeclared secret: ', str(context.exception))

    async def test_secret_string_no_external_name_in_declared_secrets(self) -> None:
        c = create_compose_mock()
        c.declared_secrets = {"my_secret_name": {"external": "true"}}
        cnt = get_minimal_container()
        cnt["_service"] = "test-service"
        cnt["secrets"] = [
            "my_secret_name",
        ]
        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--secret",
                "my_secret_name",
                "busybox",
            ],
        )

    async def test_secret_string_options_external_name_in_declared_secrets(self) -> None:
        c = create_compose_mock()
        c.declared_secrets = {
            "my_secret_name": {
                "external": "true",
                "name": "my_secret_name",
            }
        }
        cnt = get_minimal_container()
        cnt["_service"] = "test-service"
        cnt["secrets"] = [
            {
                "source": "my_secret_name",
                "target": "my_secret_name",
                "uid": "103",
                "gid": "103",
                "mode": "400",
            }
        ]

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--secret",
                "my_secret_name,uid=103,gid=103,mode=400,target=my_secret_name",
                "busybox",
            ],
        )

    async def test_secret_string_external_name_in_declared_secrets_does_not_match_secret(
        self,
    ) -> None:
        c = create_compose_mock()
        c.declared_secrets = {
            "my_secret_name": {
                "external": "true",
                "name": "wrong_secret_name",
            }
        }
        cnt = get_minimal_container()
        cnt["_service"] = "test-service"
        cnt["secrets"] = [
            "my_secret_name",
        ]

        with self.assertRaises(ValueError) as context:
            await container_to_args(c, cnt)
        self.assertIn('ERROR: Custom name/target reference ', str(context.exception))

    async def test_secret_target_does_not_match_secret_name_secret_type_mount(self) -> None:
        c = create_compose_mock()
        c.declared_secrets = {
            "my_secret_name": {
                "external": "true",
            }
        }
        cnt = get_minimal_container()
        cnt["_service"] = "test-service"
        cnt["secrets"] = [
            {
                "source": "my_secret_name",
                "target": "/tmp/custom_path",
                "type": "mount",
            }
        ]

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--secret",
                "my_secret_name,type=mount,target=/tmp/custom_path",
                "busybox",
            ],
        )

    async def test_secret_target_does_not_match_secret_name_secret_type_env(self) -> None:
        c = create_compose_mock()
        c.declared_secrets = {
            "my_secret_name": {
                "external": "true",
            }
        }
        cnt = get_minimal_container()
        cnt["_service"] = "test-service"
        cnt["secrets"] = [
            {"source": "my_secret_name", "target": "does_not_equal_secret_name", "type": "env"}
        ]

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--secret",
                "my_secret_name,type=env,target=does_not_equal_secret_name",
                "busybox",
            ],
        )

    async def test_secret_target_matches_secret_name_secret_type_not_env(self) -> None:
        c = create_compose_mock()
        c.declared_secrets = {
            "my_secret_name": {
                "external": "true",
            }
        }
        cnt = get_minimal_container()
        cnt["_service"] = "test-service"
        cnt["secrets"] = [{"source": "my_secret_name", "target": "my_secret_name", "type": "mount"}]

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--secret",
                "my_secret_name,type=mount,target=my_secret_name",
                "busybox",
            ],
        )

    @parameterized.expand([
        (
            "no_secret_target",
            {
                "file_secret": {
                    "file": "./my_secret",
                }
            },
            "file_secret",
            repo_root() + "/test_dirname/my_secret:/run/secrets/file_secret:ro,rprivate,rbind",
        ),
        (
            "relabel",
            {"file_secret": {"file": "./my_secret", "x-podman.relabel": "Z"}},
            "file_secret",
            repo_root() + "/test_dirname/my_secret:/run/secrets/file_secret:ro,rprivate,rbind,Z",
        ),
        (
            "relabel",
            {"file_secret": {"file": "./my_secret", "x-podman.relabel": "z"}},
            "file_secret",
            repo_root() + "/test_dirname/my_secret:/run/secrets/file_secret:ro,rprivate,rbind,z",
        ),
        (
            "custom_target_name",
            {
                "file_secret": {
                    "file": "./my_secret",
                }
            },
            {
                "source": "file_secret",
                "target": "custom_name",
            },
            repo_root() + "/test_dirname/my_secret:/run/secrets/custom_name:ro,rprivate,rbind",
        ),
        (
            "no_custom_target_name",
            {
                "file_secret": {
                    "file": "./my_secret",
                }
            },
            {
                "source": "file_secret",
            },
            repo_root() + "/test_dirname/my_secret:/run/secrets/file_secret:ro,rprivate,rbind",
        ),
        (
            "custom_location",
            {
                "file_secret": {
                    "file": "./my_secret",
                }
            },
            {
                "source": "file_secret",
                "target": "/etc/custom_location",
            },
            repo_root() + "/test_dirname/my_secret:/etc/custom_location:ro,rprivate,rbind",
        ),
    ])
    async def test_file_secret(
        self,
        test_name: str,
        declared_secrets: dict,
        add_to_minimal_container: dict,
        expected_volume_ref: str,
    ) -> None:
        c = create_compose_mock()
        c.declared_secrets = declared_secrets
        cnt = get_minimal_container()
        cnt["_service"] = "test-service"
        cnt["secrets"] = [add_to_minimal_container]
        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--volume",
                expected_volume_ref,
                "busybox",
            ],
        )

    async def test_file_secret_unused_params_warning(self) -> None:
        c = create_compose_mock()
        c.declared_secrets = {
            "file_secret": {
                "file": "./my_secret",
            }
        }
        cnt = get_minimal_container()
        cnt["_service"] = "test-service"
        cnt["secrets"] = [
            {
                "source": "file_secret",
                "target": "unused_params_warning",
                "uid": "103",
                "gid": "103",
                "mode": "400",
            }
        ]
        with self.assertLogs() as cm:
            args = await container_to_args(c, cnt)
        self.assertEqual(len(cm.output), 1)
        self.assertIn('with uid, gid, or mode.', cm.output[0])
        self.assertIn('unused_params_warning', cm.output[0])

        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--volume",
                repo_root()
                + "/test_dirname/my_secret:/run/secrets/unused_params_warning:ro,rprivate,rbind",
                "busybox",
            ],
        )

    async def test_environment_secret_run_raises(self) -> None:
        c = create_compose_mock()
        c.declared_secrets = {'my_secret': {'environment': 'MY_VAR'}}
        cnt = get_minimal_container()
        cnt['_service'] = 'test-service'
        cnt['secrets'] = ['my_secret']
        with self.assertRaises(ValueError) as context:
            await container_to_args(c, cnt)
        self.assertIn('not supported for runtime secrets', str(context.exception))

    async def test_external_secret_with_target_path_uid_gid_mode(self) -> None:
        """Reproduce the reported bug: external secret with custom target path, uid, gid, mode."""
        c = create_compose_mock()
        c.declared_secrets = {"ssh-private-key": {"external": True}}
        cnt = get_minimal_container()
        cnt["_service"] = "alpine_secret"
        cnt["secrets"] = [
            {
                "source": "ssh-private-key",
                "target": "/tmp/private_key",
                "mode": "700",
                "uid": "600",
                "gid": "600",
            }
        ]

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--secret",
                "ssh-private-key,uid=600,gid=600,mode=700,target=/tmp/private_key",
                "busybox",
            ],
        )

    async def test_external_secret_with_mount_type_and_target(self) -> None:
        """External secret with explicit type=mount and a target path."""
        c = create_compose_mock()
        c.declared_secrets = {"my_secret": {"external": True}}
        cnt = get_minimal_container()
        cnt["_service"] = "test-service"
        cnt["secrets"] = [
            {
                "source": "my_secret",
                "target": "/etc/secrets/my_secret",
                "type": "mount",
                "uid": "1000",
                "gid": "1000",
                "mode": "400",
            }
        ]

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--secret",
                "my_secret,uid=1000,gid=1000,mode=400,type=mount,target=/etc/secrets/my_secret",
                "busybox",
            ],
        )

    async def test_external_secret_no_target(self) -> None:
        """External secret without target should not include target in args."""
        c = create_compose_mock()
        c.declared_secrets = {"my_secret": {"external": True}}
        cnt = get_minimal_container()
        cnt["_service"] = "test-service"
        cnt["secrets"] = [
            {
                "source": "my_secret",
            }
        ]

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--secret",
                "my_secret",
                "busybox",
            ],
        )

    async def test_external_secret_custom_name_mismatch_still_raises(self) -> None:
        """External secret with mismatched 'name' should still raise ValueError."""
        c = create_compose_mock()
        c.declared_secrets = {
            "my_secret": {
                "external": True,
                "name": "different_name",
            }
        }
        cnt = get_minimal_container()
        cnt["_service"] = "test-service"
        cnt["secrets"] = ["my_secret"]

        with self.assertRaises(ValueError) as context:
            await container_to_args(c, cnt)
        self.assertIn('ERROR: Custom name/target reference ', str(context.exception))



# SPDX-License-Identifier: GPL-2.0


"""Test how secrets in files are passed to podman."""

import os
import subprocess
import unittest

from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path():
    """ "Returns the path to the compose file used for this test module"""
    return os.path.join(test_path(), "build_secrets")


class TestComposeBuildSecrets(unittest.TestCase):
    def test_run_secret(self):
        """podman run should receive file secrets as --volume

        See build_secrets/docker-compose.yaml for secret names and mount points (aka targets)

        """
        cmd = (
            "coverage",
            "run",
            podman_compose_path(),
            "--dry-run",
            "--verbose",
            "-f",
            os.path.join(compose_yaml_path(), "docker-compose.yaml"),
            "run",
            "test",
        )
        p = subprocess.run(
            cmd, stdout=subprocess.PIPE, check=False, stderr=subprocess.STDOUT, text=True
        )
        self.assertEqual(p.returncode, 0)
        secret_path = os.path.join(compose_yaml_path(), "my_secret")
        self.assertIn(f"--volume {secret_path}:/run/secrets/run_secret:ro,rprivate,rbind", p.stdout)
        self.assertIn(f"--volume {secret_path}:/tmp/run_secret2:ro,rprivate,rbind", p.stdout)

    def test_build_secret(self):
        """podman build should receive secrets as --secret, so that they can be used inside the
        Dockerfile in "RUN --mount=type=secret ..." commands.

        """
        cmd = (
            "coverage",
            "run",
            podman_compose_path(),
            "--dry-run",
            "--verbose",
            "-f",
            os.path.join(compose_yaml_path(), "docker-compose.yaml"),
            "build",
        )
        p = subprocess.run(
            cmd, stdout=subprocess.PIPE, check=False, stderr=subprocess.STDOUT, text=True
        )
        self.assertEqual(p.returncode, 0)
        secret_path = os.path.join(compose_yaml_path(), "my_secret")
        # file-sourced secrets
        self.assertIn(f"--secret id=build_secret_file,src={secret_path}", p.stdout)
        self.assertIn(f"--secret id=build_secret_file2,src={secret_path}", p.stdout)
        # environment-sourced secrets
        self.assertIn("--secret id=build_secret_env,env=MY_SECRET_VAR", p.stdout)
        self.assertIn("--secret id=build_secret_env2,env=MY_SECRET_VAR", p.stdout)

    def test_invalid_build_secret(self):
        """build secrets in docker-compose file can only have a target argument without directory
        component

        """
        cmd = (
            "coverage",
            "run",
            podman_compose_path(),
            "--dry-run",
            "--verbose",
            "-f",
            os.path.join(compose_yaml_path(), "docker-compose.yaml.invalid"),
            "build",
        )
        p = subprocess.run(
            cmd, stdout=subprocess.PIPE, check=False, stderr=subprocess.STDOUT, text=True
        )
        self.assertNotEqual(p.returncode, 0)
        self.assertIn(
            'ValueError: ERROR: Build secret "build_secret" has invalid target "/build_secret"',
            p.stdout,
        )



# SPDX-License-Identifier: GPL-2.0


"""Test how additional contexts are passed to podman."""

import os
import subprocess
import unittest

from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    """ "Returns the path to the compose file used for this test module"""
    return os.path.join(test_path(), "additional_contexts", "project")


class TestComposeBuildAdditionalContexts(unittest.TestCase):
    def test_build_additional_context(self) -> None:
        """podman build should receive additional contexts as --build-context

        See additional_context/project/docker-compose.yaml for context paths
        """
        cmd = (
            "coverage",
            "run",
            podman_compose_path(),
            "--dry-run",
            "--verbose",
            "-f",
            os.path.join(compose_yaml_path(), "docker-compose.yml"),
            "build",
        )
        p = subprocess.run(
            cmd,
            stdout=subprocess.PIPE,
            check=False,
            stderr=subprocess.STDOUT,
            text=True,
        )
        self.assertEqual(p.returncode, 0)
        self.assertIn("--build-context=data=../data_for_dict", p.stdout)
        self.assertIn("--build-context=data=../data_for_list", p.stdout)



# SPDX-License-Identifier: GPL-2.0
# pylint: disable=protected-access
from __future__ import annotations

import argparse
import os
import unittest
from typing import Any

import yaml

from podman_compose import PodmanCompose


class TestIncludeDict(unittest.TestCase):
    def setUp(self) -> None:
        self.test_files: list[str] = []

    def tearDown(self) -> None:
        for f in self.test_files:
            if os.path.exists(f):
                os.remove(f)

    def write_yaml(self, name: str, content: dict[str, Any]) -> None:
        with open(name, "w", encoding="utf-8") as f:
            yaml.safe_dump(content, f)
        self.test_files.append(name)

    def test_parse_compose_file_with_include_dict(self) -> None:
        # 1. Create a base compose file to be included
        include_content = {"services": {"base_web": {"image": "nginx:alpine"}}}
        self.write_yaml("test-compose-include.yaml", include_content)

        # 2. Create a main compose file with include detailed format (dictionary)
        main_content = {
            "version": "3.8",
            "include": [{"path": "./test-compose-include.yaml"}],
            "services": {"web": {"image": "alpine:latest"}},
        }
        self.write_yaml("test-compose-main.yaml", main_content)

        podman_compose = PodmanCompose()
        podman_compose.global_args = argparse.Namespace()
        podman_compose.global_args.file = ["test-compose-main.yaml"]
        podman_compose.global_args.project_name = "test_project"
        podman_compose.global_args.env_file = None
        podman_compose.global_args.profile = []
        podman_compose.global_args.in_pod = "false"
        podman_compose.global_args.pod_args = None
        podman_compose.global_args.no_normalize = True

        # It should run successfully without raising a TypeError
        podman_compose._parse_compose_file()

        self.assertIn("web", podman_compose.services)
        self.assertIn("base_web", podman_compose.services)



# SPDX-License-Identifier: GPL-2.0

import json
import os
import shutil
import unittest

from parameterized import parameterized

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path(test_ref_folder: str) -> str:
    return os.path.join(test_path(), "vol", test_ref_folder, "docker-compose.yml")


class TestComposeVolShortSyntax(unittest.TestCase, RunSubprocessMixin):
    @parameterized.expand([
        (True, "test1"),
        (False, "test2"),
    ])
    def test_source_host_dir(self, source_host_dir_exists: bool, service_name: str) -> None:
        project_dir = os.path.join(test_path(), "vol", "short_syntax")
        # create host source directory for volume
        if source_host_dir_exists:
            os.mkdir(os.path.join(project_dir, "test_dir"))
        try:
            self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path("short_syntax"),
                    "up",
                    "-d",
                    f"{service_name}",
                ],
                0,
            )
            # command of the service creates a new directory on mounted directory 'test_dir' in
            # container. Check if host directory now has the same directory. It represents a
            # successful mount.
            # On service test2 source host directory is created as it did not exist
            self.assertTrue(os.path.isdir(os.path.join(project_dir, "test_dir/new_dir")))

        finally:
            self.run_subprocess([
                podman_compose_path(),
                "-f",
                compose_yaml_path("short_syntax"),
                "down",
                "-t",
                "0",
            ])
            shutil.rmtree(os.path.join(project_dir, "test_dir"))

    def test_volume_source_is_symlink(self) -> None:
        try:
            base = "/tmp/symlink-test"
            real_dir = os.path.join(base, "real")
            link_dir = os.path.join(base, "link")

            os.makedirs(real_dir, exist_ok=True)

            os.symlink(real_dir, link_dir)

            self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path("short_syntax"),
                    "up",
                    "-d",
                ],
                0,
            )

            out, _ = self.run_subprocess_assert_returncode(
                [
                    "podman",
                    "inspect",
                    "short_syntax_test3_1",
                ],
                0,
            )

            container_info = json.loads(out)
            bind_string = container_info[0].get("HostConfig", {}).get("Binds")[0]
            # podman-compose should not dereference the symlink directory
            self.assertEqual(bind_string.split(":", 1)[0], link_dir)
        finally:
            self.run_subprocess([
                podman_compose_path(),
                "-f",
                compose_yaml_path("short_syntax"),
                "down",
                "-t",
                "0",
            ])
            shutil.rmtree(base, ignore_errors=True)



# SPDX-License-Identifier: GPL-2.0

import json
import os
import unittest

from packaging import version

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import get_podman_version
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path(compose_name: str) -> str:
    """ "Returns the path to the compose file used for this test module"""
    base_path = os.path.join(test_path(), "merge/volumes_merge/")
    return os.path.join(base_path, compose_name)


class TestComposeVolumesMerge(unittest.TestCase, RunSubprocessMixin):
    @unittest.skipIf(get_podman_version() >= version.parse("5.7.0"), "Breaks as of podman-5.7.1.")
    def test_volumes_merge(self) -> None:
        # test if additional compose file overrides host path and access mode of a volume
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path("docker-compose.yaml"),
                "-f",
                compose_yaml_path("docker-compose.override.yaml"),
                "up",
                "-d",
            ])

            out, _ = self.run_subprocess_assert_returncode([
                "podman",
                "exec",
                "-ti",
                "volumes_merge_web_1",
                "cat",
                "/var/www/html/index.html",
                "/var/www/html/index2.html",
                "/var/www/html/index3.html",
            ])
            self.assertEqual(
                out,
                b"The file from docker-compose.override.yaml\r\n"
                b"The file from docker-compose.override.yaml\r\n"
                b"The file from docker-compose.override.yaml\r\n",
            )

            out, _ = self.run_subprocess_assert_returncode([
                "podman",
                "inspect",
                "volumes_merge_web_1",
            ])
            volumes_info = json.loads(out.decode('utf-8'))[0]
            binds_info = volumes_info["HostConfig"]["Binds"]
            binds_info.sort()

            file_path = os.path.join(test_path(), "merge/volumes_merge/override.txt")
            expected = [
                f'{file_path}:/var/www/html/index.html:ro,rprivate,rbind',
                f'{file_path}:/var/www/html/index2.html:rw,rprivate,rbind',
                f'{file_path}:/var/www/html/index3.html:rw,rprivate,rbind',
            ]
            self.assertEqual(binds_info, expected)
        finally:
            out, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path("docker-compose.yaml"),
                "-f",
                compose_yaml_path("docker-compose.override.yaml"),
                "down",
                "-t",
                "0",
            ])



import unittest

from parameterized import parameterized

from podman_compose import PodmanCompose
from podman_compose import get_net_args
from tests.unit.test_container_to_args import create_compose_mock

PROJECT_NAME = "test_project_name"
SERVICE_NAME = "service_name"
CONTAINER_NAME = f"{PROJECT_NAME}_{SERVICE_NAME}_1"


def get_networked_compose(num_networks: int = 1) -> PodmanCompose:
    compose = create_compose_mock(PROJECT_NAME)
    for network in range(num_networks):
        compose.networks[f"net{network}"] = {
            "driver": "bridge",
            "ipam": {
                "config": [
                    {"subnet": f"192.168.{network}.0/24"},
                    {"subnet": f"fd00:{network}::/64"},
                ]
            },
            "enable_ipv6": True,
        }

    if num_networks == 1:
        compose.default_net = "net0"

    return compose


def get_minimal_container() -> dict:
    return {
        "name": CONTAINER_NAME,
        "service_name": SERVICE_NAME,
        "image": "busybox",
    }


class TestGetNetArgs(unittest.TestCase):
    def test_minimal(self) -> None:
        compose = get_networked_compose()
        container = get_minimal_container()

        self.assertEqual(
            get_net_args(compose, container),
            [f"--network={PROJECT_NAME}_net0:alias={SERVICE_NAME}"],
        )

    def test_default_net_is_None(self) -> None:
        compose = get_networked_compose()
        container = get_minimal_container()

        mac_address = "11:22:33:44:55:66"
        container["mac_address"] = mac_address

        compose.default_net = None

        self.assertEqual(
            get_net_args(compose, container),
            [f"--network=bridge:alias={SERVICE_NAME},mac={mac_address}"],
        )

    def test_one_net(self) -> None:
        compose = get_networked_compose()
        container = get_minimal_container()
        container["networks"] = {"net0": {}}

        self.assertEqual(
            get_net_args(compose, container),
            [f"--network={PROJECT_NAME}_net0:alias={SERVICE_NAME}"],
        )

    def test_alias(self) -> None:
        compose = get_networked_compose()
        container = get_minimal_container()
        container["networks"] = {"net0": {}}
        container["_aliases"] = ["alias1", "alias2"]

        self.assertEqual(
            get_net_args(compose, container),
            [f"--network={PROJECT_NAME}_net0:alias={SERVICE_NAME},alias=alias1,alias=alias2"],
        )

    def test_aliases_on_network_scope(self) -> None:
        compose = get_networked_compose()
        container = get_minimal_container()
        container["networks"] = {"net0": {"aliases": ["alias1"]}}

        self.assertEqual(
            get_net_args(compose, container),
            [f"--network={PROJECT_NAME}_net0:alias={SERVICE_NAME},alias=alias1"],
        )

    def test_one_ipv4(self) -> None:
        ip = "192.168.0.42"
        compose = get_networked_compose()
        container = get_minimal_container()
        container["networks"] = {"net0": {"ipv4_address": ip}}

        self.assertEqual(
            get_net_args(compose, container),
            [f"--network={PROJECT_NAME}_net0:ip={ip},alias={SERVICE_NAME}"],
        )

    def test_one_ipv6(self) -> None:
        ipv6_address = "fd00:0::42"
        compose = get_networked_compose()
        container = get_minimal_container()
        container["networks"] = {"net0": {"ipv6_address": ipv6_address}}

        self.assertEqual(
            get_net_args(compose, container),
            [f"--network={PROJECT_NAME}_net0:ip6={ipv6_address},alias={SERVICE_NAME}"],
        )

    def test_one_mac(self) -> None:
        mac = "00:11:22:33:44:55"
        compose = get_networked_compose()
        container = get_minimal_container()
        container["networks"] = {"net0": {}}
        container["mac_address"] = mac

        self.assertEqual(
            get_net_args(compose, container),
            [f"--network={PROJECT_NAME}_net0:mac={mac},alias={SERVICE_NAME}"],
        )

    def test_one_mac_two_nets(self) -> None:
        mac = "00:11:22:33:44:55"
        compose = get_networked_compose(num_networks=6)
        container = get_minimal_container()
        container["networks"] = {"net0": {}, "net1": {}}
        container["mac_address"] = mac

        self.assertEqual(
            get_net_args(compose, container),
            [
                f"--network={PROJECT_NAME}_net0:mac={mac},alias={SERVICE_NAME}",
                f"--network={PROJECT_NAME}_net1:alias={SERVICE_NAME}",
            ],
        )

    @parameterized.expand([
        "mac_address",
        "x-podman.mac_address",
    ])
    def test_mac_on_network(self, mac_attr: str) -> None:
        mac = "00:11:22:33:44:55"
        compose = get_networked_compose()
        container = get_minimal_container()
        container["networks"] = {"net0": {mac_attr: mac}}

        self.assertEqual(
            get_net_args(compose, container),
            [f"--network={PROJECT_NAME}_net0:mac={mac},alias={SERVICE_NAME}"],
        )

    def test_two_nets_as_dict(self) -> None:
        compose = get_networked_compose(num_networks=2)
        container = get_minimal_container()
        container["networks"] = {"net0": {}, "net1": {}}

        self.assertEqual(
            get_net_args(compose, container),
            [
                f"--network={PROJECT_NAME}_net0:alias={SERVICE_NAME}",
                f"--network={PROJECT_NAME}_net1:alias={SERVICE_NAME}",
            ],
        )

    def test_two_nets_as_list(self) -> None:
        compose = get_networked_compose(num_networks=2)
        container = get_minimal_container()
        container["networks"] = ["net0", "net1"]

        self.assertEqual(
            get_net_args(compose, container),
            [
                f"--network={PROJECT_NAME}_net0:alias={SERVICE_NAME}",
                f"--network={PROJECT_NAME}_net1:alias={SERVICE_NAME}",
            ],
        )

    def test_two_ipv4(self) -> None:
        ip0 = "192.168.0.42"
        ip1 = "192.168.1.42"
        compose = get_networked_compose(num_networks=2)
        container = get_minimal_container()
        container["networks"] = {"net0": {"ipv4_address": ip0}, "net1": {"ipv4_address": ip1}}

        self.assertEqual(
            get_net_args(compose, container),
            [
                f"--network={PROJECT_NAME}_net0:ip={ip0},alias={SERVICE_NAME}",
                f"--network={PROJECT_NAME}_net1:ip={ip1},alias={SERVICE_NAME}",
            ],
        )

    def test_two_ipv6(self) -> None:
        ip0 = "fd00:0::42"
        ip1 = "fd00:1::42"
        compose = get_networked_compose(num_networks=2)
        container = get_minimal_container()
        container["networks"] = {"net0": {"ipv6_address": ip0}, "net1": {"ipv6_address": ip1}}

        self.assertEqual(
            get_net_args(compose, container),
            [
                f"--network={PROJECT_NAME}_net0:ip6={ip0},alias={SERVICE_NAME}",
                f"--network={PROJECT_NAME}_net1:ip6={ip1},alias={SERVICE_NAME}",
            ],
        )

    # custom extension; not supported by docker-compose
    def test_two_mac(self) -> None:
        mac0 = "00:00:00:00:00:01"
        mac1 = "00:00:00:00:00:02"
        compose = get_networked_compose(num_networks=2)
        container = get_minimal_container()
        container["networks"] = {
            "net0": {"x-podman.mac_address": mac0},
            "net1": {"x-podman.mac_address": mac1},
        }

        self.assertEqual(
            get_net_args(compose, container),
            [
                f"--network={PROJECT_NAME}_net0:mac={mac0},alias={SERVICE_NAME}",
                f"--network={PROJECT_NAME}_net1:mac={mac1},alias={SERVICE_NAME}",
            ],
        )

    def test_mixed_mac(self) -> None:
        ip4_0 = "192.168.0.42"
        ip4_1 = "192.168.1.42"
        ip4_2 = "192.168.2.42"
        mac_0 = "00:00:00:00:00:01"
        mac_1 = "00:00:00:00:00:02"

        compose = get_networked_compose(num_networks=3)
        container = get_minimal_container()
        container["networks"] = {
            "net0": {"ipv4_address": ip4_0},
            "net1": {"ipv4_address": ip4_1, "x-podman.mac_address": mac_0},
            "net2": {"ipv4_address": ip4_2},
        }
        container["mac_address"] = mac_1

        expected_exception = (
            r"specifying mac_address on both container and network level is not supported"
        )
        self.assertRaisesRegex(RuntimeError, expected_exception, get_net_args, compose, container)

    def test_mixed_config(self) -> None:
        ip4_0 = "192.168.0.42"
        ip4_1 = "192.168.1.42"
        ip6_0 = "fd00:0::42"
        ip6_2 = "fd00:2::42"
        mac = "00:11:22:33:44:55"
        compose = get_networked_compose(num_networks=4)
        container = get_minimal_container()
        container["networks"] = {
            "net0": {"ipv4_address": ip4_0, "ipv6_address": ip6_0},
            "net1": {"ipv4_address": ip4_1},
            "net2": {"ipv6_address": ip6_2},
            "net3": {},
        }
        container["mac_address"] = mac

        self.assertEqual(
            get_net_args(compose, container),
            [
                f"--network={PROJECT_NAME}_net0:ip={ip4_0},ip6={ip6_0},mac={mac},alias={SERVICE_NAME}",
                f"--network={PROJECT_NAME}_net1:ip={ip4_1},alias={SERVICE_NAME}",
                f"--network={PROJECT_NAME}_net2:ip6={ip6_2},alias={SERVICE_NAME}",
                f"--network={PROJECT_NAME}_net3:alias={SERVICE_NAME}",
            ],
        )

    @parameterized.expand([
        ("bridge", [f"--network=bridge:alias={SERVICE_NAME},mac=11:22:33:44:55:66"]),
        (
            "bridge:ip=10.88.0.3",
            [f"--network=bridge:ip=10.88.0.3,alias={SERVICE_NAME},mac=11:22:33:44:55:66"],
        ),
        ("host", ["--network=host"]),
        ("none", ["--network=none"]),
        ("slirp4netns", ["--network=slirp4netns"]),
        ("slirp4netns:cidr=10.42.0.0/24", ["--network=slirp4netns:cidr=10.42.0.0/24"]),
        ("private", ["--network=private"]),
        ("pasta", ["--network=pasta"]),
        ("pasta:--ipv4-only,-a,10.0.2.0", ["--network=pasta:--ipv4-only,-a,10.0.2.0"]),
        ("ns:my_namespace", ["--network=ns:my_namespace"]),
        ("container:my_container", ["--network=container:my_container"]),
    ])
    def test_network_modes(self, network_mode: str, expected_args: list) -> None:
        compose = get_networked_compose()
        container = get_minimal_container()
        container["network_mode"] = network_mode

        mac_address = "11:22:33:44:55:66"
        container["network_mode"] = network_mode
        container["mac_address"] = mac_address

        self.assertEqual(get_net_args(compose, container), expected_args)

    def test_network_mode_invalid(self) -> None:
        compose = get_networked_compose()
        container = get_minimal_container()
        container["network_mode"] = "invalid_mode"

        with self.assertRaises(SystemExit):
            get_net_args(compose, container)

    def test_network__mode_service(self) -> None:
        compose = get_networked_compose()
        compose.container_names_by_service = {
            "service_1": ["container_1"],
            "service_2": ["container_2"],
        }

        container = get_minimal_container()
        container["network_mode"] = "service:service_2"

        self.assertEqual(get_net_args(compose, container), ["--network=container:container_2"])



# SPDX-License-Identifier: GPL-2.0

import os
import unittest
from subprocess import PIPE
from subprocess import Popen

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(os.path.join(test_path(), "secrets"), "docker-compose.yaml")


class TestComposeNoSecrets(unittest.TestCase, RunSubprocessMixin):
    created_secrets = [
        "podman_compose_test_secret",
        "podman_compose_test_secret_2",
        "podman_compose_test_secret_3",
        "podman_compose_test_secret_custom_name",
    ]

    def setUp(self) -> None:
        for secret in self.created_secrets:
            p = Popen(["podman", "secret", "create", secret, "-"], stdin=PIPE)
            p.communicate(secret.encode('utf-8'))

    def tearDown(self) -> None:
        for secret in self.created_secrets:
            self.run_subprocess_assert_returncode([
                "podman",
                "secret",
                "rm",
                f"{secret}",
            ])

    # test if secrets are saved and available in respective files of a container
    def test_secrets(self) -> None:
        try:
            _, error, _ = self.run_subprocess(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "up",
                    "test",
                ],
            )

            self.assertNotIn(
                b'That is un-supported and a no-op and is ignored.',
                error,
            )
            self.assertIn(
                b'WARNING: Service test uses secret unused_params_warning with uid, '
                + b'gid, or mode. These fields are not supported by this implementation '
                + b'of the Compose file',
                error,
            )

            output, _ = self.run_subprocess_assert_returncode(["podman", "logs", "secrets_test_1"])
            expected_output = (
                b'/run/secrets/custom_name:important-secret-is-important\n'
                + b'/run/secrets/file_secret:important-secret-is-important\n'
                + b'/run/secrets/podman_compose_test_secret:podman_compose_test_secret\n'
                + b'/run/secrets/podman_compose_test_secret_3:podman_compose_test_secret_3\n'
                + b'/run/secrets/unused_params_warning:important-secret-is-important\n'
                + b'important-secret-is-important\n'
                + b'podman_compose_test_secret\n'
            )
            self.assertEqual(expected_output, output)
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
                "-t",
                "0",
            ])



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from packaging import version

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import get_podman_version


def base_path() -> str:
    """Returns the base path for the project"""
    return os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))


def test_path() -> str:
    """Returns the path to the tests directory"""
    return os.path.join(base_path(), "tests/integration")


def podman_compose_path() -> str:
    """Returns the path to the podman compose script"""
    return os.path.join(base_path(), "podman_compose.py")


def is_root() -> bool:
    return os.geteuid() == 0


def failure_exitcode_when_rootful() -> int:
    if is_root():
        return 125
    return 0


# If a compose file has userns_mode set, setting in_pod to True, results in error.
# Default in_pod setting is True, unless compose file provides otherwise.
# Compose file provides custom in_pod option, which can be overridden by command line in_pod option.
# Test all combinations of command line argument in_pod and compose file argument in_pod.
class TestPodmanComposeInPod(unittest.TestCase, RunSubprocessMixin):
    # compose file provides x-podman in_pod=false
    @unittest.skipIf(
        get_podman_version() > version.parse("4.4.0"), "Breaks as of podman-4.9.5 and podman-5.4.2."
    )
    def test_x_podman_in_pod_false_command_line_in_pod_not_exists(self) -> None:
        """
        Test that podman-compose will not create a pod, when x-podman in_pod=false and command line
        does not provide this option
        """
        command_up = [
            "python3",
            os.path.join(base_path(), "podman_compose.py"),
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_false",
                "docker-compose.yml",
            ),
            "up",
            "-d",
        ]

        down_cmd = [
            "python3",
            podman_compose_path(),
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_false",
                "docker-compose.yml",
            ),
            "down",
        ]

        try:
            self.run_subprocess_assert_returncode(command_up, failure_exitcode_when_rootful())

        finally:
            self.run_subprocess_assert_returncode(down_cmd)
            command_rm_pod = ["podman", "pod", "rm", "pod_custom_x-podman_false"]
            # throws an error, can not actually find this pod because it was not created
            self.run_subprocess_assert_returncode(command_rm_pod, expected_returncode=1)

    def test_x_podman_in_pod_false_command_line_in_pod_true(self) -> None:
        """
        Test that podman-compose does not allow pod creating even with command line in_pod=True
        when --userns and --pod are set together: throws an error
        """
        # FIXME: creates a pod anyway, although it should not
        command_up = [
            "python3",
            os.path.join(base_path(), "podman_compose.py"),
            "--in-pod=True",
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_false",
                "docker-compose.yml",
            ),
            "up",
            "-d",
        ]

        try:
            out, err = self.run_subprocess_assert_returncode(command_up, 125)
            self.assertEqual(b"Error: --userns and --pod cannot be set together" in err, True)

        finally:
            command_rm_pod = ["podman", "pod", "rm", "pod_custom_x-podman_false"]
            # should throw an error of not being able to find this pod (because it should not have
            # been created) and have expected_returncode=1 (see FIXME above)
            self.run_subprocess_assert_returncode(command_rm_pod)

    @unittest.skipIf(
        get_podman_version() > version.parse("4.4.0"), "Breaks as of podman-4.9.5 and podman-5.4.2."
    )
    def test_x_podman_in_pod_false_command_line_in_pod_false(self) -> None:
        """
        Test that podman-compose will not create a pod as command line sets in_pod=False
        """
        command_up = [
            "python3",
            os.path.join(base_path(), "podman_compose.py"),
            "--in-pod=False",
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_false",
                "docker-compose.yml",
            ),
            "up",
            "-d",
        ]

        down_cmd = [
            "python3",
            podman_compose_path(),
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_false",
                "docker-compose.yml",
            ),
            "down",
        ]

        try:
            self.run_subprocess_assert_returncode(command_up, failure_exitcode_when_rootful())

        finally:
            self.run_subprocess_assert_returncode(down_cmd)
            command_rm_pod = ["podman", "pod", "rm", "pod_custom_x-podman_false"]
            # can not actually find this pod because it was not created
            self.run_subprocess_assert_returncode(command_rm_pod, 1)

    @unittest.skipIf(
        get_podman_version() > version.parse("4.4.0"), "Breaks as of podman-4.9.5 and podman-5.4.2."
    )
    def test_x_podman_in_pod_false_command_line_in_pod_empty_string(self) -> None:
        """
        Test that podman-compose will not create a pod, when x-podman in_pod=false and command line
        command line in_pod=""
        """
        command_up = [
            "python3",
            os.path.join(base_path(), "podman_compose.py"),
            "--in-pod=",
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_false",
                "docker-compose.yml",
            ),
            "up",
            "-d",
        ]

        down_cmd = [
            "python3",
            podman_compose_path(),
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_false",
                "docker-compose.yml",
            ),
            "down",
        ]

        try:
            self.run_subprocess_assert_returncode(command_up, failure_exitcode_when_rootful())

        finally:
            self.run_subprocess_assert_returncode(down_cmd)
            command_rm_pod = ["podman", "pod", "rm", "pod_custom_x-podman_false"]
            # can not actually find this pod because it was not created
            self.run_subprocess_assert_returncode(command_rm_pod, 1)

    # compose file provides x-podman in_pod=true
    def test_x_podman_in_pod_true_command_line_in_pod_not_exists(self) -> None:
        """
        Test that podman-compose does not allow pod creating when --userns and --pod are set
        together even when x-podman in_pod=true: throws an error
        """
        # FIXME: creates a pod anyway, although it should not
        # Container is not created, so command 'down' is not needed
        command_up = [
            "python3",
            os.path.join(base_path(), "podman_compose.py"),
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_true",
                "docker-compose.yml",
            ),
            "up",
            "-d",
        ]

        try:
            out, err = self.run_subprocess_assert_returncode(command_up, 125)
            self.assertEqual(b"Error: --userns and --pod cannot be set together" in err, True)

        finally:
            command_rm_pod = ["podman", "pod", "rm", "pod_custom_x-podman_true"]
            # should throw an error of not being able to find this pod (it should not have been
            # created) and have expected_returncode=1 (see FIXME above)
            self.run_subprocess_assert_returncode(command_rm_pod)

    def test_x_podman_in_pod_true_command_line_in_pod_true(self) -> None:
        """
        Test that podman-compose does not allow pod creating when --userns and --pod are set
        together even when x-podman in_pod=true and command line in_pod=True: throws an error
        """
        # FIXME: creates a pod anyway, although it should not
        # Container is not created, so command 'down' is not needed
        command_up = [
            "python3",
            os.path.join(base_path(), "podman_compose.py"),
            "--in-pod=True",
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_true",
                "docker-compose.yml",
            ),
            "up",
            "-d",
        ]

        try:
            out, err = self.run_subprocess_assert_returncode(command_up, 125)
            self.assertEqual(b"Error: --userns and --pod cannot be set together" in err, True)

        finally:
            command_rm_pod = ["podman", "pod", "rm", "pod_custom_x-podman_true"]
            # should throw an error of not being able to find this pod (because it should not have
            # been created) and have expected_returncode=1 (see FIXME above)
            self.run_subprocess_assert_returncode(command_rm_pod)

    @unittest.skipIf(
        get_podman_version() > version.parse("4.4.0"), "Breaks as of podman-4.9.5 and podman-5.4.2."
    )
    def test_x_podman_in_pod_true_command_line_in_pod_false(self) -> None:
        """
        Test that podman-compose will not create a pod as command line sets in_pod=False
        """
        command_up = [
            "python3",
            os.path.join(base_path(), "podman_compose.py"),
            "--in-pod=False",
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_true",
                "docker-compose.yml",
            ),
            "up",
            "-d",
        ]

        down_cmd = [
            "python3",
            podman_compose_path(),
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_true",
                "docker-compose.yml",
            ),
            "down",
        ]

        try:
            self.run_subprocess_assert_returncode(command_up, failure_exitcode_when_rootful())

        finally:
            self.run_subprocess_assert_returncode(down_cmd)
            command_rm_pod = ["podman", "pod", "rm", "pod_custom_x-podman_false"]
            # can not actually find this pod because it was not created
            self.run_subprocess_assert_returncode(command_rm_pod, 1)

    def test_x_podman_in_pod_true_command_line_in_pod_empty_string(self) -> None:
        """
        Test that podman-compose does not allow pod creating when --userns and --pod are set
        together even when x-podman in_pod=true and command line in_pod="": throws an error
        """
        # FIXME: creates a pod anyway, although it should not
        # Container is not created, so command 'down' is not needed
        command_up = [
            "python3",
            os.path.join(base_path(), "podman_compose.py"),
            "--in-pod=",
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_true",
                "docker-compose.yml",
            ),
            "up",
            "-d",
        ]

        try:
            out, err = self.run_subprocess_assert_returncode(command_up, 125)
            self.assertEqual(b"Error: --userns and --pod cannot be set together" in err, True)

        finally:
            command_rm_pod = ["podman", "pod", "rm", "pod_custom_x-podman_true"]
            # should throw an error of not being able to find this pod (because it should not have
            # been created) and have expected_returncode=1 (see FIXME above)
            self.run_subprocess_assert_returncode(command_rm_pod)

    # compose file does not provide x-podman in_pod
    def test_x_podman_in_pod_not_exists_command_line_in_pod_not_exists(self) -> None:
        """
        Test that podman-compose does not allow pod creating when --userns and --pod are set
        together: throws an error
        """
        # FIXME: creates a pod anyway, although it should not
        # Container is not created, so command 'down' is not needed
        command_up = [
            "python3",
            os.path.join(base_path(), "podman_compose.py"),
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_not_exists",
                "docker-compose.yml",
            ),
            "up",
            "-d",
        ]

        try:
            out, err = self.run_subprocess_assert_returncode(command_up, 125)
            self.assertEqual(b"Error: --userns and --pod cannot be set together" in err, True)

        finally:
            command_rm_pod = ["podman", "pod", "rm", "pod_custom_x-podman_not_exists"]
            # should throw an error of not being able to find this pod (it should not have been
            # created) and have expected_returncode=1 (see FIXME above)
            self.run_subprocess_assert_returncode(command_rm_pod)

    def test_x_podman_in_pod_not_exists_command_line_in_pod_true(self) -> None:
        """
        Test that podman-compose does not allow pod creating when --userns and --pod are set
        together even when x-podman in_pod=true: throws an error
        """
        # FIXME: creates a pod anyway, although it should not
        # Container was not created, so command 'down' is not needed
        command_up = [
            "python3",
            os.path.join(base_path(), "podman_compose.py"),
            "--in-pod=True",
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_not_exists",
                "docker-compose.yml",
            ),
            "up",
            "-d",
        ]

        try:
            out, err = self.run_subprocess_assert_returncode(command_up, 125)
            self.assertEqual(b"Error: --userns and --pod cannot be set together" in err, True)

        finally:
            command_rm_pod = ["podman", "pod", "rm", "pod_custom_x-podman_not_exists"]
            # should throw an error of not being able to find this pod (because it should not have
            # been created) and have expected_returncode=1 (see FIXME above)
            self.run_subprocess_assert_returncode(command_rm_pod)

    @unittest.skipIf(
        get_podman_version() > version.parse("4.4.0"), "Breaks as of podman-4.9.5 and podman-5.4.2."
    )
    def test_x_podman_in_pod_not_exists_command_line_in_pod_false(self) -> None:
        """
        Test that podman-compose will not create a pod as command line sets in_pod=False
        """
        command_up = [
            "python3",
            os.path.join(base_path(), "podman_compose.py"),
            "--in-pod=False",
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_not_exists",
                "docker-compose.yml",
            ),
            "up",
            "-d",
        ]

        down_cmd = [
            "python3",
            podman_compose_path(),
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_not_exists",
                "docker-compose.yml",
            ),
            "down",
        ]

        try:
            self.run_subprocess_assert_returncode(command_up, failure_exitcode_when_rootful())

        finally:
            self.run_subprocess_assert_returncode(down_cmd)

            command_rm_pod = ["podman", "pod", "rm", "pod_custom_x-podman_not_exists"]
            # can not actually find this pod because it was not created
            self.run_subprocess_assert_returncode(command_rm_pod, 1)

    @unittest.skipIf(
        get_podman_version() > version.parse("4.4.0"), "Breaks as of podman-4.9.5 and podman-5.4.2."
    )
    def test_x_podman_in_pod_not_exists_command_line_in_pod_not_exists_docker_compat(self) -> None:
        """
        Test that podman-compose will not create a pod when docker compat is requested.
        """
        command_up = [
            "python3",
            os.path.join(base_path(), "podman_compose.py"),
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_not_exists",
                "docker-compose.yml",
            ),
            "up",
            "-d",
        ]

        down_cmd = [
            "python3",
            podman_compose_path(),
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_not_exists",
                "docker-compose.yml",
            ),
            "down",
        ]

        env = {
            "PODMAN_COMPOSE_DOCKER_COMPOSE_COMPAT": "1",
        }

        try:
            self.run_subprocess_assert_returncode(
                command_up, failure_exitcode_when_rootful(), env=env
            )

        finally:
            self.run_subprocess_assert_returncode(down_cmd, env=env)

            command_rm_pod = ["podman", "pod", "rm", "pod_custom_x-podman_not_exists"]
            # can not actually find this pod because it was not created
            self.run_subprocess_assert_returncode(command_rm_pod, 1)

    @unittest.skipIf(
        get_podman_version() > version.parse("4.4.0"), "Breaks as of podman-4.9.5 and podman-5.4.2."
    )
    def test_x_podman_in_pod_not_exists_command_line_in_pod_not_exists_env_var(self) -> None:
        """
        Test that podman-compose will not create a pod when env var is set.
        """
        command_up = [
            "python3",
            os.path.join(base_path(), "podman_compose.py"),
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_not_exists",
                "docker-compose.yml",
            ),
            "up",
            "-d",
        ]

        down_cmd = [
            "python3",
            podman_compose_path(),
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_not_exists",
                "docker-compose.yml",
            ),
            "down",
        ]

        env = {
            "PODMAN_COMPOSE_IN_POD": "0",
        }

        try:
            self.run_subprocess_assert_returncode(
                command_up, failure_exitcode_when_rootful(), env=env
            )

        finally:
            self.run_subprocess_assert_returncode(down_cmd, env=env)

            command_rm_pod = ["podman", "pod", "rm", "pod_custom_x-podman_not_exists"]
            # can not actually find this pod because it was not created
            self.run_subprocess_assert_returncode(command_rm_pod, 1)

    def test_x_podman_in_pod_custom_name(self) -> None:
        """
        Test that podman-compose will create a pod with a custom name
        """
        command_up = [
            "python3",
            os.path.join(base_path(), "podman_compose.py"),
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_custom_name",
                "docker-compose.yml",
            ),
            "up",
            "--no-start",
        ]

        try:
            self.run_subprocess_assert_returncode(command_up)

        finally:
            command_rm_pod = ["podman", "pod", "rm", "custom_test_pod_name"]
            self.run_subprocess_assert_returncode(command_rm_pod)

    def test_x_podman_in_pod_not_exists_command_line_in_pod_empty_string(self) -> None:
        """
        Test that podman-compose does not allow pod creating when --userns and --pod are set
        together: throws an error
        """
        # FIXME: creates a pod anyway, although it should not
        # Container was not created, so command 'down' is not needed
        command_up = [
            "python3",
            os.path.join(base_path(), "podman_compose.py"),
            "--in-pod=",
            "-f",
            os.path.join(
                base_path(),
                "tests",
                "integration",
                "in_pod",
                "custom_x-podman_not_exists",
                "docker-compose.yml",
            ),
            "up",
            "-d",
        ]

        try:
            out, err = self.run_subprocess_assert_returncode(command_up, 125)
            self.assertEqual(b"Error: --userns and --pod cannot be set together" in err, True)

        finally:
            command_rm_pod = ["podman", "pod", "rm", "pod_custom_x-podman_not_exists"]
            # should throw an error of not being able to find this pod (because it should not have
            # been created) and have expected_returncode=1 (see FIXME above)
            self.run_subprocess_assert_returncode(command_rm_pod)



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(
        test_path(), "merge/reset_and_override_tags/reset_tag_service/docker-compose.yaml"
    )


class TestComposeResetTagService(unittest.TestCase, RunSubprocessMixin):
    # test if whole service from docker-compose.yaml file is reset
    def test_reset_tag_service(self) -> None:
        reset_file = os.path.join(
            test_path(),
            "merge/reset_and_override_tags/reset_tag_service/docker-compose.reset_service.yaml",
        )
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "-f",
                reset_file,
                "up",
            ])

            # app service was fully reset in docker-compose.reset_tag_service.yaml file, therefore
            # does not exist. A new service was created instead.
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "-f",
                reset_file,
                "ps",
            ])
            self.assertNotIn(b"reset_tag_service_app_1", output)
            self.assertIn(b"reset_tag_service_app2_1", output)

            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "-f",
                reset_file,
                "logs",
                "--no-log-prefix",
                "--no-color",
            ])
            self.assertEqual(output, b"One\n")
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0

import json
import os
import unittest
from typing import Any

from packaging import version
from parameterized import parameterized

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import get_podman_version
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path(scenario: str) -> str:
    return os.path.join(
        os.path.join(test_path(), "compose_up_behavior"), f"docker-compose_{scenario}.yaml"
    )


class TestComposeUpBehavior(unittest.TestCase, RunSubprocessMixin):
    def get_existing_containers(self, scenario: str) -> dict[str, Any]:
        out, _ = self.run_subprocess_assert_returncode(
            [
                podman_compose_path(),
                "-f",
                compose_yaml_path(scenario),
                "ps",
                "--format",
                'json',
            ],
        )
        containers = json.loads(out)
        return {
            c.get("Names")[0]: {
                "name": c.get("Names")[0],
                "id": c.get("Id"),
                "service_name": c.get("Labels", {}).get("io.podman.compose.service", ""),
                "config_hash": c.get("Labels", {}).get("io.podman.compose.config-hash", ""),
                "exited": c.get("Exited"),
            }
            for c in containers
        }

    @parameterized.expand([
        (
            "service_change_app",
            "service_change_base",
            ["up"],
            {"app"},
        ),
        (
            "service_change_app",
            "service_change_base",
            ["up", "app"],
            {"app"},
        ),
        (
            "service_change_app",
            "service_change_base",
            ["up", "db"],
            set(),
        ),
        (
            "service_change_db",
            "service_change_base",
            ["up"],
            {"db", "app"},
        ),
        (
            "service_change_db",
            "service_change_base",
            ["up", "app"],
            {"db", "app"},
        ),
        (
            "service_change_db",
            "service_change_base",
            ["up", "db"],
            {"db", "app"},
        ),
    ])
    def test_recreate_on_config_changed(
        self,
        change_to: str,
        running_scenario: str,
        command_args: list[str],
        expect_recreated_services: set[str],
    ) -> None:
        try:
            self.run_subprocess_assert_returncode(
                [podman_compose_path(), "-f", compose_yaml_path(running_scenario), "up", "-d"],
            )

            original_containers = self.get_existing_containers(running_scenario)

            out, err = self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "--verbose",
                    "-f",
                    compose_yaml_path(change_to),
                    *command_args,
                    "-d",
                ],
            )

            new_containers = self.get_existing_containers(change_to)
            recreated_services = {
                c.get("service_name")
                for c in original_containers.values()
                if new_containers.get(c.get("name"), {}).get("id") != c.get("id")
            }

            self.assertEqual(
                recreated_services,
                expect_recreated_services,
                msg=f"Expected services to be recreated: {expect_recreated_services}, "
                f"but got: {recreated_services}, containers: "
                f"[{original_containers}, {new_containers}]",
            )
            self.assertTrue(
                all([c.get("exited") is False for c in new_containers.values()]),
                msg="Not all containers are running after up command",
            )

        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(change_to),
                "down",
                "-t",
                "0",
            ])

    @unittest.skipIf(
        get_podman_version() < version.parse("5.6.0"),
        "The image pull policy feature was only added as of Podman 5.6.0.",
    )
    def test_pull_only_if_image_missing(self) -> None:
        """Verify image is pulled because default pull policy is --missing"""

        compose_file = compose_yaml_path("default_pull_policy")
        image = "docker.io/library/alpine:latest"

        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_file,
                "up",
                "-d",
            ])

            # Remove image while container is still up
            self.run_subprocess_assert_returncode([
                "podman",
                "rmi",
                "-f",
                image,
            ])

            # up container one more time, it sees missing images and pulls them again
            _, error = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "--verbose",
                "-f",
                compose_file,
                "up",
                "-d",
            ])

            # podman pull command is explicitly run before container teardown and now is
            # visible in --verbose output. After container teardown, podman create is called
            # and already has needed images available
            self.assertIn(b"podman pull --policy missing alpine:latest", error)

            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_file,
                "ps",
            ])
            self.assertIn(b"compose_up_behavior_test_1", output)
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_file,
                "down",
                "-t",
                "0",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "rmi",
                "-f",
                image,
            ])

    @unittest.skipIf(
        get_podman_version() < version.parse("5.6.0"),
        "The image pull policy feature was only added as of Podman 5.6.0.",
    )
    def test_pull_default_policy_overrides_lower_priority_policy(self) -> None:
        """Verify pull policy flag with higher priority overrides the default pull
        policy --missing"""

        compose_file = compose_yaml_path("override_missing")
        image = "docker.io/library/alpine:latest"

        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_file,
                "up",
                "-d",
            ])

            # Remove image while container is still up
            self.run_subprocess_assert_returncode([
                "podman",
                "rmi",
                "-f",
                image,
            ])

            # up container one more time
            _, error = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "--verbose",
                "-f",
                compose_file,
                "up",
                "-d",
            ])
            # default pull-policy is --missing, but it has been overridden by policy of
            # higher priority --always
            self.assertIn(b"podman pull --policy always alpine:latest", error)
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_file,
                "down",
                "-t",
                "0",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "rmi",
                "-f",
                image,
            ])

    @unittest.skipIf(
        get_podman_version() < version.parse("5.6.0"),
        "The image pull policy feature was only added as of Podman 5.6.0.",
    )
    def test_localhost_image_not_pulled(self) -> None:
        """Verify existing localhost/ images are used locally without pulling"""

        compose_file = compose_yaml_path("non_existent_localhost_image")
        image = "localhost/test-image:1"

        try:
            # pre-create image locally by tagging a minimal base image,
            # this simulates the image already existing locally
            self.run_subprocess_assert_returncode(["podman", "pull", "alpine:latest"])
            self.run_subprocess_assert_returncode(["podman", "tag", "alpine:latest", image])

            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_file,
                "up",
                "-d",
            ])

            # Remove image while container is still up
            self.run_subprocess_assert_returncode([
                "podman",
                "rmi",
                "-f",
                image,
            ])

            # since the compose file has both 'build' and 'image', podman-compose
            # should use the existing image (or rebuild if necessary), but it should
            # never attempt to pull localhost/ images
            _, error = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "--verbose",
                "-f",
                compose_file,
                "up",
                "-d",
            ])
            self.assertNotIn(b"Trying to pull", error)

            # Confirm container is actually started with localhost/ image
            output, _ = self.run_subprocess_assert_returncode([
                "podman",
                "ps",
                "-a",
                "--filter",
                f"ancestor={image}",
                "--format",
                "{{.Image}}",
            ])
            self.assertIn(b"localhost/test-image:1", output)
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_file,
                "down",
            ])
            self.run_subprocess_assert_returncode([
                "podman",
                "rmi",
                "-f",
                image,
            ])

    @unittest.skipIf(
        get_podman_version() < version.parse("5.6.0"),
        "The image pull policy feature was only added as of Podman 5.6.0.",
    )
    def test_localhost_image_built_if_does_not_exist(self) -> None:
        """Verify non-existent localhost/ images are built instead of pulling"""

        compose_file = compose_yaml_path("build_localhost_image")
        image = "localhost/not-exists"

        # check image does not exist
        self.run_subprocess_assert_returncode(
            [
                "podman",
                "image",
                "exists",
                image,
            ],
            1,
        )

        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_file,
                "up",
                "-d",
            ])

            # Remove image while container is still up
            self.run_subprocess_assert_returncode([
                "podman",
                "rmi",
                "-f",
                image,
            ])

            # image was not pulled, it was built
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "--verbose",
                "-f",
                compose_file,
                "up",
                "-d",
            ])
            self.assertNotIn(b"Trying to pull", output)

            # After up command, image now exists (was built)
            output, _ = self.run_subprocess_assert_returncode([
                "podman",
                "images",
                "--filter",
                f"reference={image}",
                "--format",
                "{{.Repository}}:{{.Tag}}",
            ])
            self.assertIn(image.encode(), output)

            # Container uses the locally built image
            output, _ = self.run_subprocess_assert_returncode([
                "podman",
                "ps",
                "-a",
                "--filter",
                f"ancestor={image}",
                "--format",
                "{{.Image}}",
            ])
            self.assertIn(image.encode(), output)
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_file,
                "down",
            ])
            self.run_subprocess_assert_returncode(
                ["podman", "rmi", "-f", image],
            )



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from parameterized import parameterized

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path(scenario: str) -> str:
    return os.path.join(
        os.path.join(test_path(), "compose_down_behavior"), f"docker-compose_{scenario}.yaml"
    )


class TestComposeDownBehavior(unittest.TestCase, RunSubprocessMixin):
    @parameterized.expand([
        ("default", ["down"], set()),
        (
            "default",
            ["down", "app"],
            {
                "compose_down_behavior_db_1",
                "compose_down_behavior_no_deps_1",
            },
        ),
        (
            "default",
            ["down", "db"],
            {
                "compose_down_behavior_no_deps_1",
            },
        ),
    ])
    def test_compose_down(
        self, scenario: str, command_args: list[str], expect_containers: set[str]
    ) -> None:
        try:
            self.run_subprocess_assert_returncode(
                [podman_compose_path(), "-f", compose_yaml_path(scenario), "up", "-d"],
            )

            self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(scenario),
                    *command_args,
                ],
            )

            out, _ = self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(scenario),
                    "ps",
                    "--format",
                    '{{ .Names }}',
                ],
            )

            actual_containers = set()
            for line in out.decode('utf-8').strip().split('\n'):
                name = line.strip()
                if name:
                    actual_containers.add(name)

            self.assertEqual(actual_containers, expect_containers)
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(scenario),
                "down",
                "-t",
                "0",
            ])



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_base_path() -> str:
    return os.path.join(test_path(), "env_file_tests")


class TestComposeEnvFile(unittest.TestCase, RunSubprocessMixin):
    def test_path_env_file_inline(self) -> None:
        # Test taking env variable value directly from env-file when its path is inline path
        base_path = compose_base_path()
        path_compose_file = os.path.join(base_path, "project/container-compose.yaml")
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "--env-file",
                os.path.join(base_path, "env-files/project-1.env"),
                "up",
            ])
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "logs",
                "--no-log-prefix",
                "--no-color",
            ])
            # takes only value ZZVAR1 as container-compose.yaml file requires
            self.assertEqual(output, b"ZZVAR1=podman-rocks-123\n")
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "down",
            ])

    def test_path_env_file_flat_in_compose_file(self) -> None:
        # Test taking env variable value from env-file/project-1.env which was declared in
        # compose file's env_file
        base_path = compose_base_path()
        path_compose_file = os.path.join(base_path, "project/container-compose.env-file-flat.yaml")
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "up",
            ])
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "logs",
                "--no-log-prefix",
                "--no-color",
            ])
            # takes all values with a substring ZZ as container-compose.env-file-flat.yaml
            # file requires
            self.assertEqual(
                output,
                b"ZZVAR1=podman-rocks-123\nZZVAR2=podman-rocks-124\nZZVAR3=podman-rocks-125\n",
            )
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "down",
            ])

    def test_path_env_file_obj_in_compose_file(self) -> None:
        # take variable value from env-file project-1.env which was declared in compose
        # file's env_file by -path: ...
        base_path = compose_base_path()
        path_compose_file = os.path.join(base_path, "project/container-compose.env-file-obj.yaml")
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "up",
            ])
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "logs",
                "--no-log-prefix",
                "--no-color",
            ])
            # takes all values with a substring ZZ as container-compose.env-file-obj.yaml
            # file requires
            self.assertEqual(
                output,
                b"ZZVAR1=podman-rocks-123\nZZVAR2=podman-rocks-124\nZZVAR3=podman-rocks-125\n",
            )
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "down",
            ])

    def test_exists_optional_env_file_path_in_compose_file(self) -> None:
        # test taking env variable values from several env-files when one of them is optional
        # and exists
        base_path = compose_base_path()
        path_compose_file = os.path.join(
            base_path, "project/container-compose.env-file-obj-optional-exists.yaml"
        )
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "up",
            ])
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "logs",
                "--no-log-prefix",
                "--no-color",
            ])
            # FIXME: gives a weird output, needs to be double checked
            self.assertEqual(
                output,
                b"ZZVAR1=podman-rocks-223\nZZVAR2=podman-rocks-224\nZZVAR3=podman-rocks-125\n",
            )
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "down",
            ])

    def test_missing_optional_env_file_path_in_compose_file(self) -> None:
        # test taking env variable values from several env-files when one of them is optional and
        # is missing (silently skip it)
        base_path = compose_base_path()
        path_compose_file = os.path.join(
            base_path, "project/container-compose.env-file-obj-optional-missing.yaml"
        )
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "up",
            ])
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "logs",
                "--no-log-prefix",
                "--no-color",
            ])
            # takes all values with a substring ZZ as container-compose.env-file-obj-optional.yaml
            # file requires
            self.assertEqual(
                output,
                b"ZZVAR1=podman-rocks-123\nZZVAR2=podman-rocks-124\nZZVAR3=podman-rocks-125\n",
            )
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "down",
            ])

    def test_var_value_inline_overrides_env_file_path_inline(self) -> None:
        # Test overriding env value when value is declared in inline command
        base_path = compose_base_path()
        path_compose_file = os.path.join(base_path, "project/container-compose.yaml")
        try:
            self.run_subprocess_assert_returncode([
                "env",
                "ZZVAR1=podman-rocks-321",
                podman_compose_path(),
                "-f",
                path_compose_file,
                "--env-file",
                os.path.join(base_path, "env-files/project-1.env"),
                "up",
            ])
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "logs",
                "--no-log-prefix",
                "--no-color",
            ])
            # takes only value ZZVAR1 as container-compose.yaml file requires
            self.assertEqual(output, b"ZZVAR1=podman-rocks-321\n")
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "down",
            ])

    def test_taking_env_variables_from_env_files_from_different_directories(self) -> None:
        # FIXME: It is not clear what this test actually tests, but from README.md it looks like:
        # Test overriding env values by directory env-files-tests/.env file values
        # and only take value from project/.env, when it does not exist in env-files-tests/.env
        base_path = compose_base_path()
        path_compose_file = os.path.join(
            base_path, "project/container-compose.load-.env-in-project.yaml"
        )
        try:
            # looks like 'run' command does not actually create a container, so output_logs can not
            # be used for test comparison
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "run",
                "--rm",
                "app",
            ])
            # takes all values with a substring ZZ as container-compose.load-.env-in-project.yaml
            # file requires
            # first line is random ID so is ignored in asserting
            lines = output.decode('utf-8').split('\n')[1:]

            self.assertEqual(
                lines,
                [
                    'ZZVAR1=This value is loaded but should be overwritten\r',
                    'ZZVAR2=This value is loaded from .env in project/ directory\r',
                    'ZZVAR3=TEST\r',
                    '',
                ],
            )
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                path_compose_file,
                "down",
            ])

    def test_env_var_value_accessed_in_compose_file_short_syntax(self) -> None:
        # Test that compose file can access the environment variable set in .env file using
        # short syntax, that is: only the name of environment variable is used in "environment:" in
        # compose.yml file and its value is picked up directly from .env file
        # long syntax of environment variables interpolation is tested in
        # tests/integration/interpolation

        base_path = compose_base_path()
        compose_file_path = os.path.join(base_path, "project/container-compose.short_syntax.yaml")
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_file_path,
                "up",
                "-d",
            ])
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_file_path,
                "logs",
                "--no-log-prefix",
                "--no-color",
            ])
            # ZZVAR3 was set in .env file
            self.assertEqual(output, b"ZZVAR3=TEST\n")
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_file_path,
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0
from __future__ import annotations

import argparse
import copy
import os
import unittest

import yaml
from parameterized import parameterized

from podman_compose import PodmanCompose


class TestCanMergeBuild(unittest.TestCase):
    @parameterized.expand([
        ({}, {}, {}),
        ({}, {"test": "test"}, {"test": "test"}),
        ({"test": "test"}, {}, {"test": "test"}),
        ({"test": "test-1"}, {"test": "test-2"}, {"test": "test-2"}),
        ({}, {"build": "."}, {"build": {"context": "."}}),
        ({"build": "."}, {}, {"build": {"context": "."}}),
        ({"build": "./dir-1"}, {"build": "./dir-2"}, {"build": {"context": "./dir-2"}}),
        ({}, {"build": {"context": "./dir-1"}}, {"build": {"context": "./dir-1"}}),
        ({"build": {"context": "./dir-1"}}, {}, {"build": {"context": "./dir-1"}}),
        (
            {"build": {"context": "./dir-1"}},
            {"build": {"context": "./dir-2"}},
            {"build": {"context": "./dir-2"}},
        ),
        (
            {},
            {"build": {"dockerfile": "dockerfile-1"}},
            {"build": {"dockerfile": "dockerfile-1"}},
        ),
        (
            {"build": {"dockerfile": "dockerfile-1"}},
            {},
            {"build": {"dockerfile": "dockerfile-1"}},
        ),
        (
            {"build": {"dockerfile": "./dockerfile-1"}},
            {"build": {"dockerfile": "./dockerfile-2"}},
            {"build": {"dockerfile": "./dockerfile-2"}},
        ),
        (
            {"build": {"dockerfile": "./dockerfile-1"}},
            {"build": {"context": "./dir-2"}},
            {"build": {"dockerfile": "./dockerfile-1", "context": "./dir-2"}},
        ),
        (
            {"build": {"dockerfile": "./dockerfile-1", "context": "./dir-1"}},
            {"build": {"dockerfile": "./dockerfile-2", "context": "./dir-2"}},
            {"build": {"dockerfile": "./dockerfile-2", "context": "./dir-2"}},
        ),
        (
            {"build": {"dockerfile": "./dockerfile-1"}},
            {"build": {"dockerfile": "./dockerfile-2", "args": ["ENV1=1"]}},
            {"build": {"dockerfile": "./dockerfile-2", "args": ["ENV1=1"]}},
        ),
        (
            {"build": {"dockerfile": "./dockerfile-2", "args": ["ENV1=1"]}},
            {"build": {"dockerfile": "./dockerfile-1"}},
            {"build": {"dockerfile": "./dockerfile-1", "args": ["ENV1=1"]}},
        ),
        (
            {"build": {"dockerfile": "./dockerfile-2", "args": ["ENV1=1"]}},
            {"build": {"dockerfile": "./dockerfile-1", "args": ["ENV2=2"]}},
            {"build": {"dockerfile": "./dockerfile-1", "args": ["ENV1=1", "ENV2=2"]}},
        ),
        (
            {"build": {"dockerfile": "./dockerfile-1", "args": {"ENV1": "1"}}},
            {"build": {"dockerfile": "./dockerfile-2", "args": {"ENV2": "2"}}},
            {"build": {"dockerfile": "./dockerfile-2", "args": ["ENV1=1", "ENV2=2"]}},
        ),
        (
            {"build": {"dockerfile": "./dockerfile-1", "args": ["ENV1=1"]}},
            {"build": {"dockerfile": "./dockerfile-2", "args": {"ENV2": "2"}}},
            {"build": {"dockerfile": "./dockerfile-2", "args": ["ENV1=1", "ENV2=2"]}},
        ),
    ])
    def test_parse_compose_file_when_multiple_composes(self, input, override, expected):
        compose_test_1 = {"services": {"test-service": input}}
        compose_test_2 = {"services": {"test-service": override}}
        dump_yaml(compose_test_1, "test-compose-1.yaml")
        dump_yaml(compose_test_2, "test-compose-2.yaml")

        podman_compose = PodmanCompose()
        set_args(podman_compose, ["test-compose-1.yaml", "test-compose-2.yaml"])

        podman_compose._parse_compose_file()  # pylint: disable=protected-access

        actual_compose = {}
        if podman_compose.services:
            actual_compose = podman_compose.original_configuration(
                podman_compose.services["test-service"]
            )
        self.assertEqual(actual_compose, expected)

    def test_parse_with_map_merge_into_none(self):
        compose_test_1 = {"volumes": {"vol_a": None}}
        compose_test_2 = {
            "volumes": {
                "vol_a": {
                    "driver_opts": {"type": "none", "device": "/dev/some", "o": "bind"},
                }
            }
        }
        expected = {"driver_opts": {"device": "/dev/some", "o": "bind", "type": "none"}}
        dump_yaml(compose_test_1, "test-compose-1.yaml")
        dump_yaml(compose_test_2, "test-compose-2.yaml")

        podman_compose = PodmanCompose()
        set_args(podman_compose, ["test-compose-1.yaml", "test-compose-2.yaml"])

        podman_compose._parse_compose_file()  # pylint: disable=protected-access

        actual_compose = podman_compose.vols["vol_a"]
        self.assertEqual(actual_compose, expected)

    # $$$ is a placeholder for either command or entrypoint
    @parameterized.expand([
        ({}, {"$$$": []}, {"$$$": []}),
        ({"$$$": []}, {}, {"$$$": []}),
        ({"$$$": []}, {"$$$": "sh-2"}, {"$$$": ["sh-2"]}),
        ({"$$$": "sh-2"}, {"$$$": []}, {"$$$": []}),
        ({}, {"$$$": "sh"}, {"$$$": ["sh"]}),
        ({"$$$": "sh"}, {}, {"$$$": ["sh"]}),
        ({"$$$": "sh-1"}, {"$$$": "sh-2"}, {"$$$": ["sh-2"]}),
        ({"$$$": ["sh-1"]}, {"$$$": "sh-2"}, {"$$$": ["sh-2"]}),
        ({"$$$": "sh-1"}, {"$$$": ["sh-2"]}, {"$$$": ["sh-2"]}),
        ({"$$$": "sh-1"}, {"$$$": ["sh-2", "sh-3"]}, {"$$$": ["sh-2", "sh-3"]}),
        ({"$$$": ["sh-1"]}, {"$$$": ["sh-2", "sh-3"]}, {"$$$": ["sh-2", "sh-3"]}),
        ({"$$$": ["sh-1", "sh-2"]}, {"$$$": ["sh-3", "sh-4"]}, {"$$$": ["sh-3", "sh-4"]}),
        ({}, {"$$$": ["sh-3", "sh      4"]}, {"$$$": ["sh-3", "sh      4"]}),
        ({"$$$": "sleep infinity"}, {"$$$": "sh"}, {"$$$": ["sh"]}),
        ({"$$$": "sh"}, {"$$$": "sleep infinity"}, {"$$$": ["sleep", "infinity"]}),
        (
            {},
            {"$$$": "bash -c 'sleep infinity'"},
            {"$$$": ["bash", "-c", "sleep infinity"]},
        ),
    ])
    def test_parse_compose_file_when_multiple_composes_keys_command_entrypoint(
        self, base_template, override_template, expected_template
    ):
        for key in ['command', 'entrypoint']:
            base, override, expected = template_to_expression(
                base_template, override_template, expected_template, key
            )
            compose_test_1 = {"services": {"test-service": base}}
            compose_test_2 = {"services": {"test-service": override}}
            dump_yaml(compose_test_1, "test-compose-1.yaml")
            dump_yaml(compose_test_2, "test-compose-2.yaml")

            podman_compose = PodmanCompose()
            set_args(podman_compose, ["test-compose-1.yaml", "test-compose-2.yaml"])

            podman_compose._parse_compose_file()  # pylint: disable=protected-access

            actual = {}
            if podman_compose.services:
                actual = podman_compose.original_configuration(
                    podman_compose.services["test-service"]
                )
            self.assertEqual(actual, expected)


def set_args(podman_compose: PodmanCompose, file_names: list[str]) -> None:
    podman_compose.global_args = argparse.Namespace()
    podman_compose.global_args.file = file_names
    podman_compose.global_args.project_name = None
    podman_compose.global_args.env_file = None
    podman_compose.global_args.profile = []
    podman_compose.global_args.in_pod = "1"
    podman_compose.global_args.pod_args = None
    podman_compose.global_args.no_normalize = True


def dump_yaml(compose: dict, name: str) -> None:
    with open(name, "w", encoding="utf-8") as outfile:
        yaml.safe_dump(compose, outfile, default_flow_style=False)


def template_to_expression(base, override, expected, key):
    base_copy = copy.deepcopy(base)
    override_copy = copy.deepcopy(override)
    expected_copy = copy.deepcopy(expected)

    expected_copy[key] = expected_copy.pop("$$$")
    if "$$$" in base:
        base_copy[key] = base_copy.pop("$$$")
    if "$$$" in override:
        override_copy[key] = override_copy.pop("$$$")
    return base_copy, override_copy, expected_copy


def test_clean_test_yamls() -> None:
    test_files = ["test-compose-1.yaml", "test-compose-2.yaml"]
    for file in test_files:
        if os.path.exists(file):
            os.remove(file)
